# monte carlo code to fit a brokem power law
import numpy as np

dat=np.loadtxt('testinput.dat')
x=dat[:,0]
y=dat[:,1]
sig=dat[:,2]


# define the model
def model(w,p0,w0,alpha):
    return( p0 / (1.+ (w/w0)**alpha ) )
model = np.vectorize(model)

nparm=3
nits=100
p=np.zeros((1,nparm))
pscale = np.zeros(nparm)
pnew=np.zeros((1,nparm))
pold=np.zeros(1,nparm))

prej=np.zeros(nparm)
ncheck =10
nrej = 4

# best guesses
p[0]=1.0
p[1]=0.01
p[2]=2
pscale[:]=0.1

pold=1.*p
pnew=1.*pold
ymod = model(x,p[0],p[1],p[2])
cisqold = ( (y - ymod) / sig )**2


for iteration in range(nits):

    for iparm in range(nparm):
        
        pold = p[iparm]
        pstep = pscale[iparm]
        
        p[iparm]=pold + myrandom.normdis(1,pold,pstep)
    
    
    
    


# script to calculate the keplarian rotation velocity of a flat disc at an input radius in ld

import numpy as np

G  = 6.673e-11
M0 = 2e30
ld = 2.59020684e13

print 'Welcome to mykeplerrot.py, The code that calculates rotation velocities and wavelength offsets, so you can check it later and find a bug in the code.'
print ''
urld=np.float( raw_input('Please enter disc radius (Ld):-') )
print ''
uembh=np.float( raw_input('PLease enter black hole mass (M0):-') )

print ''


# Temperature info ##################################################

rs=3000*uembh
r0rs=3.0
r0=r0rs*rs

urrs=urld*ld/rs

apower=0.75
bpower=0.75
pi=np.pi
emdot=1.0
year=365*24*3600
udmdt=M0/year
s=5.67e-8
c=2.9979e8
eta=0.1
fourpi=4.*pi
zx=3.0 * rs

T0v4=3/(8*pi)*10**(np.log10(G*uembh)+np.log10(emdot)-3*np.log10(r0)+np.log10(M0)+np.log10(udmdt)-np.log10(s))
T0x4=1/fourpi*10**(np.log10(zx)+np.log10(eta*c*c)+np.log10(emdot)+np.log10(udmdt)-3*np.log10(r0)-np.log10(s))

T=np.sqrt(np.sqrt(T0v4*(r0rs/urrs)**(bpower*4)+T0x4*(r0rs/urrs)**(apower*4)))
wav = 2.89711e-3/T *1e10 # peak wavelength at the input radius (in Angstroms)

# end of temperature calculation ##################################################



print ''
print ''

vkms = np.sqrt( G*uembh*M0/ (ld*urld) ) /1000
dwav = wav*vkms/(c/1000)

print 'Rotation velocity (kms-1):- ',vkms
print 'Emitted wavelength (Ang):- ',wav
print 'dwav:- ',dwav







#python code to take x,y,sigy return bet fit with uncertainty envelopes
import numpy as np

def myoptav(y,sigy):
    
    sigy2=sigy*sigy
    
    top=(y/sigy2).sum()
    bot=(1/sigy2).sum()
    optav=top/bot
    return(optav)
    



def mylinfitenv(x,y,sigy):

    xbar=myoptav(x,sigy)

# intercept b (optimal average)

    top=0.0
    bot=0.0
    si2=sigy*sigy
    temp=1/si2
    bot=temp.sum()
    top=(y*temp).sum()
    b=top/bot
    sigb=np.sqrt(1/bot)
    
    #print b,sigb,'b and sig b mylinfit.py'
    #bo=raw_input()
# slope (optimal scaling of patter)

    xsubbar=x-xbar
    temp=xsubbar/si2

    bot=(temp*xsubbar).sum()
    top=((y-b)*temp).sum()


    a=top/bot
    siga=np.sqrt(1/bot)

    #print a,siga,top, bot, xbar,'a and sig, top, bot, a mylinfit.py'

    
    #bo=raw_input()
#!!! this gets you the orthogonal slope(a, siga) but the intercept is just the optimal average of points
#! in the y direction. To get a reliable value for the intercept, need to find out where the curve
#! hits the axis intercept = ybar - (xbar * a)  where ybar = old intetcept b 
#! the above is in Keiths ada notes lecture 8, the below is written down on some long lost scrap of paper.
   
    ybar=b
    
    b = b - a*xbar
    
    return(b,sigb,a,siga,xbar,ybar)
    
    
    
    
# end of function





#
#x=np.arange(10)+1
#y=1.*x
#sigy=np.ones(10)*0.1
#y[3]=4.2
#y[6]=6.8
#
#dat=mylinfitenv(x,y,sigy)
#
#print dat






import numpy as np
from mylcgen import *
from myrandom import *
import matplotlib.pylab as plt
import myresample as mrs

#generate test light curve and add noise
datpre      = mylcgen(tlo=0,thi=100,dt=0.03,iseed=34245)

 

ndat     = np.shape(datpre[:,0])[0]
datmean  = np.std(datpre[:,1])
sig      = np.ones(ndat)/10*datmean

dat = mrs.myresample(dir='',fname=[''],dtave=2.0,sampmin=0.8,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],sig)).T)
ndat = np.shape(dat[:,0])[0]
sig = dat[:,2]


for i in range(ndat):
 dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
y        = dat[:,1] +50
time     = dat[:,0]

tlo = np.min(time)
thi = np.max(time)
dt  = np.mean(time[1:]-time[:-1])

#input some minimum and maximum frequency range for the Fourier terms
flo = 0.5/(thi-tlo)
fhi = 2./dt

#angular frequency
w = np.arange(flo,fhi+flo,flo)*2*np.pi
w = np.arange(0,fhi+flo,flo)*2*np.pi
w[0] = flo/50


#now compute the hessian matrix
nw = np.shape(w)[0]
np2 = 2*nw

cvec = np.ones(np2)
hes = np.ones((np2,np2))
sig2 = sig*sig
#save the sine and cosine value in  2d array
cwt = np.ones((ndat,nw))
swt = np.ones((ndat,nw))
for iw in range(nw):
 cwt[:,iw] = np.cos(w[iw]*time)
 swt[:,iw] = np.sin(w[iw]*time)

iwp = 0
for ikp in range(0,np2,2):
 wp = w[iwp]
 
 wp2 = wp
 cvec[ikp]   = np.sum(y*cwt[:,iwp]/sig2)
 cvec[ikp+1] = np.sum(y*swt[:,iwp]/sig2)
 print iwp,nw
 iw = 0
 for ik in range(0,np2,2):
  #cosine column
  hnow_c = np.sum(cwt[:,iw]*cwt[:,iwp]/sig2)
  hnow_s = np.sum(swt[:,iw]*cwt[:,iwp]/sig2)
  if (ik == ikp):
   hnow_c = hnow_c + wp2
  hes[ikp,ik]   = hnow_c
  hes[ikp,ik+1] = hnow_s 
   
  #sine column
  hnow_c = np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
  hnow_s = np.sum(swt[:,iw]*swt[:,iwp]/sig2)
  if (ik == ikp):
   hnow_s = hnow_s + wp2
  hes[ikp+1,ik]   = hnow_c
  hes[ikp+1,ik+1] = hnow_s 
  iw = iw + 1
  
 iwp = iwp + 1
      
#invert the matrix to get the covariance matrix
cov = np.linalg.inv(hes)


#find the parameters
parm  = cov.dot(cvec)
skout = parm[1::2]
ckout = parm[::2]










#plot the input and output light curves to see if they match
freq = w/2/np.pi
dtres = dt/10
tplot = np.arange(tlo,thi+dtres,dtres)
xplot = []
for tnow in tplot:
 xplot.append( np.sum(ckout*np.cos(w*tnow) + skout*np.sin(w*tnow)) )

fig = plt.figure()
ax1 = fig.add_subplot(211) 
ax1.errorbar(time,y,sig,ls='',color='k')
ax1.plot(tplot,xplot,color='b')
ax1.plot(tplot,ckout[0]*np.cos(w[0]*tplot) + skout[0]*np.sin(w[0]*tplot))
ax1.set_xlabel('Time (days)')
ax1.set_ylabel('flux')


ax2 = fig.add_subplot(212) 
ax2.plot(freq,ckout**2 + skout**2,ls='',marker='o')
ax2.plot(freq,skout**2,ls='',color='g',marker='o')
ax2.plot(freq,ckout**2,ls='',color='b',marker='o')
ax2.set_xscale('log')
ax2.set_yscale('log')
ax2.set_xlabel('Frequency (cyc/day)')
ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
ax2.grid(True)
plt.savefig('test_r_hesfit.pdf')











    


# redistribute noise on gaussian light curve
import numpy as np
import os

pwd=os.getcwd()

dir='../fort/fortcode/fakelc/'
os.chdir(dir)

		
	
fname=['resamp_fake_3556.51_nonoise.dat','resamp_fake_4702.48_nonoise.dat','resamp_fake_6175.57_nonoise.dat','rebinned_fake_6300.00_nonoise.dat','resamp_fake_7489.97_nonoise.dat','resamp_fake_8946.70_nonoise.dat']
nfile=len(fname)

for i in range(nfile):
    dat=np.loadtxt(fname[i])
    n=dat[:,0].shape[0]
    dat[:,1]=dat[:,1]+np.random.randn(n)*dat[:,2]

    np.savetxt('refuxed_'+fname[i],dat)


os.chdir(pwd)


#David Starkey UIUC 08/02/2018
#code to use sigma clipping algorithm to reject outliers from a randomw walk model fit
#import myfitrw_current as mfw
import myfitrw_simp as mfw
#import astropy.stats as apys
from pylab import *
from mylcgen import *

def mscrw(x,y,sig=[],niteration=10,sigmaclip = 5):

 xin = 1.*x
 nx = np.shape(x)[0]
 idxinclude = np.arange(nx)
 tdat = x[idxinclude]
 ydat = y[idxinclude]
 
 #if no error bars just treat everything equally
 if (sig == []):
  sigdat = np.ones(nx)
 else:
  sigdat = sig[idxinclude]
 
 #this bit tells the code to use the errors on the model rather than the data errorbars if
 #your data has no error bars
 if (sig == []):
  nits = 100
 else:
  nits = 1
  
  
 for i in range(niteration):
  
  nxold = np.shape(sigdat)[0]
  op = mfw.fitrw([tdat],[ydat],[sigdat],floin=-1,fhiin=-1,ploton=1,dtresin=-1,nits = nits)
  ymod = op[-2]
  ymodsd = op[-1]
  yin = np.abs(ydat - ymod)
  
  if (sig == []):
   idinc = np.where(yin < sigmaclip*ymodsd)
  else:
   idinc = np.where(yin < sigmaclip*sigdat)
   
  #clf()
  #
  #
  #errorbar(tdat,ydat,sigdat,ls=None)
  #plot(tdat,ymod)
  #title('sdfsdfs')
  #print idinc
  for i in range(np.shape(yin)[0]):
   print ydat[i],ymod[i],ymodsd[i]
  show()
  
  #a = apys.sigma_clip(yin, sigma=sigmaclip, sigma_lower=None, sigma_upper=None, iters=1, axis=None, copy=True)
  #idinc = ~a.mask
  tdat = tdat[idinc]
  ydat = ydat[idinc]
  sigdat = sigdat[idinc]
  
  nxnew = np.shape(sigdat)[0]
 
  print 'sigclip report'
  print 'nold',nxold
  print 'nnew',nxnew
  print 'rejected',nxold-nxnew,'points'
  if (nxold == nxnew):
   break
 return(tdat,ydat,sigdat)
 
 
 
 
 

 
 
 
 
 
 
##!!!!!!!!!!!!!!!!!!! test the code using this program
#
#





#generate test light curve and add noise
cadence = 3.0
timeall = []
sigall  = []
yall    = []
datpre      = mylcgen(tlo=0,thi=100,dt=cadence,iseed=132423)
npre     = np.shape(datpre[:,0])[0]
sig = np.std(datpre[:,1])/10
#add noise
sigpre = np.ones(npre)*sig
datpre[:,1] = np.random.randn(npre)*sig + datpre[:,1]
#add bad data to test sigma clipping
datpre[10,1] = datpre[10,1] + 100*sig
datpre[5,1] = datpre[5,1] - 40*sig

timeall = datpre[:,0] + np.random.randn(npre)*0.1#npre,datpre[:,0],0.1)
yall = datpre[:,1]
sigall = sigpre





#if your data has error bars, put these in as the sigma argument
#a = mscrw(timeall,yall,sig = sigall,niteration=3,sigmaclip = 2)

#if your data has no error bars use the error snakes on the fitted model in the sigma clip
a = mscrw(timeall,yall,niteration=1,sigmaclip = 5)




#make a seperate plot of old and new data
clf()
scatter(timeall,yall)
scatter(a[0],a[1])

show()
datmean  = np.std(datpre[:,1])
snow =  np.ones(npre)/10*datmean 
#dat = #mrs.myresample(dir='',fname=[''],dtave=2.0,sampmin=0.8,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],snow)).T)
ndat = np.shape(dat[:,0])[0]
sig = dat[:,2]
for i in range(ndat):
 dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
sigall.append( sig )
yall.append( dat[:,1] +50 )
timeall.append( dat[:,0] )



a = fitrw(timeall,yall,sigall,ploton=1) 



#
#     
 


#code to make fake light curves
#makes driver
#convolves with response function
#adds noise
#resamples appropriately

#INPUT wav[...nwav], snr[...nwav] lists of wavelengths and desired snr for the light curves
#..... dtmeanin[...nwav] list of the mean cadence for each light curve
#..... embh, degi, thi the balck hole mass, inclination and length of the light curve (starts at t = 0)

#OPTIONAL INPUT dl, z, er, emdot the luminosity distance in Mpc, redshift, eddington ratio and accretion rate emdot
#if emdot is -ve, use er (eddington ratio) to calculate mdot appropriate for this er.
#if emdot +ve ignore eddington ratio (er)

#iseed set to -1 to automatically generate driving light curve. Can manually set to a large positive
#...integer to get the same driving light curve each call


#OUTPUT tlcout[...nt], xlcout[...nt], echosave[...nt,...nwav],sigsave[...nt,...nwav] 
# 1 and 2-d lists of the timegrid of the driver and echolight curfves respectively
# no errors calculated for driving light curve include later if needed

import numpy as np
import pylab as plt
import os
import astropy_stark.mytfb_quick as tfb
import astropy_stark.myedlum as me
import astropy_stark.myconvolve as mc
import astropy_stark.mydisksim as mds
import astropy_stark.myfake_amp as mfa
import astropy_stark.myrandom as mr
import astropy_stark.mylcgen as mlg
import astropy_stark.myresample as mrs


def myfake(wavin, snr, dtmeanin, embh = 1.e7, degi = 0.0,
           thi = 100.0,gap = [], dlMpc = 70., z = 0.0,
           er=0.1, emdot=-1, eta = 0.1, dtres = 0.05,
           tauhi = 50., diag_plot = 0, iseed = -1,sampmin=0.8,
           meanforcein=[],sdforcein=[],sigforcein = [],
           mean_x = 0., sd_x = 1.,thfwhm = 0.2,
           thcent = 1.0,fakeplotnorm = 1,affine=0,
           paraffine=[-1,-2,-3,-4],
           dirsave = None,
           naffine=1000,noise_gap=0,
           tfx=0,T0v=1.e4,T0x=1.e4,sv=0.75,sx=0.75,
           verbose = False):
 '''
 generate fake light curve using a random walk driver and covolving with the accretion disc response fucntion
 described in Starkey et al 2016 'https://academic.oup.com/mnras/article-abstract/456/2/1960/1066664?redirectedFrom=PDF'
 :param wavin: list of wavelengths
 :param snr: list of signal to noise of each disk light curve as a relative to light curve standard deviation
 :param dtmeanin: average cadence for each response light curve
 OPTIONAL ARGUMENTS
 :param dtmeanin: mean cadence default = 1 day
 :param embh: black hole mass default = 1.e7
 :param degi: disk inlincation (default 0)
 :param thi: light curve time span (default = 100 days)
 :param gap:
 :param dlMpc:
 :param z:
 :param er:
 :param emdot:
 :param eta:
 :param dtres:
 :param tauhi:
 :param diag_plot:
 :param iseed: set to be a positive integer e.g 3423452 to always get the same results from the function call,
 else will get a new random light curve each time
 :param sampmin:
 :param meanforcein:
 :param sdforcein:
 :param sigforcein:
 :param mean_x:
 :param sd_x:
 :param thfwhm:
 :param thcent:
 :param fakeplotnorm:
 :param pycall:
 :param affine:
 :param paraffine:
 :param dirsave: if not None then save all output light curves to this directory in the
 directory structure required by the
 fortran version of cream. Will also general the required creamnames.dat file within the directory
 :param naffine:
 :param noise_gap:
 :param tfx:
 :param T0v:
 :param T0x:
 :param sv:
 :param sx:
 :return: output (dictionary of driver, echo light curve and response fn's)
 '''

 if dirsave is not None:
  os.system('rm -rf '+dirsave)
  os.system('mkdir '+dirsave)


 if (wavin == -1):
  wav = [-4000.0]
 else:
  wav = wavin
  
 nwav = len(wav)
 if (len(meanforcein) == 0): 
  meanforce = [-1]*nwav
 else:
  meanforce = list(meanforcein)
  
 if (len(sdforcein) == 0):
  sdforce = [-1]*nwav
 else:
  sdforce = list(sdforcein) 

 if (len(sigforcein) == 0):
  sigforce = [-1]*nwav
 else:
  sigforce = list(sigforcein)
  
 echosave = []
 sigsave  = []
 psi_out  = []
 if (emdot < 0):
  emdot = me.ermin_mdotout(embh,er,eta=eta)
 #print 'what is coming out here i dont understand',emdot
 
 #obtain disk spectrum
 fnu = []
 idw = 0
 for wavnow in wav:
  if (meanforce[idw] == -1): 
   fnunow = mds.mds([np.abs(wavnow)],embh,emdot,degi, dl = dlMpc, radlosch=3.0,radhisch=10000., ngrid = 1000, mjy = 1)
  else:
   fnunow = meanforce[idw]
  fnu.append(fnunow)
 fnu = np.array(fnu)  
 
 #generate fake light curve
 dat = mlg.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=thi+tauhi+2*dtres,dt=dtres,ploton=0,iseed=iseed,sdnorm = 1., meannorm = 0.)
 tlc = dat[:,0]
 xlc = dat[:,1]
 
 taunow = np.arange(0,tauhi,dtres)
 ntau   = np.shape(taunow)[0]
 idxlo  = np.where(tlc > tauhi)[0][0]#chop off lower limit where lookback is below the start time of the light curve
 
 tlclo  = tlc[idxlo]
 tlcout = tlc[idxlo:] #- tlclo
 xlcout = xlc[idxlo:]
 
 
 
 
 
 
 #define a list t_out to store the light curve times resampled to simulate uneven - random sampling
 t_out = []
 
 
 ntlc = np.shape(tlc[idxlo:])[0]
 for i in range(nwav):
  wavnow = wav[i]
  
  #what to do if we want the driver out
  if (wavnow == 0):
   echolc = 1.*xlc[idxlo:]
   sdwav = sd_x
   fdiskmean = mean_x
   psinow = np.zeros(ntau)
  else:
   
   #calculate expected mean
   if (meanforce[i] == -1):
    fdiskmean = fnu[i]
   else:
    fdiskmean = meanforce[i]
   #calculate expected variability amplitude
   if (sdforce[i] == -1):
    if fdiskmean == 0:
     fdfm = 1.0
    else:
     fdfm = fdiskmean
    sdwav = mfa.mfamp(embh,np.abs(wavnow),fdfm,thi, dMpc = dlMpc, z= z)
   else:
    sdwav = sdforce[i]
   
   #generate response function
   #print 'before response',i,wavnow,fdiskmean,emdot,embh,wavnow,'response function info',pycall
   psinow = tfb.pytfb_sub(taunow,embh,emdot,wavnow, degi, t0vin=-1, t0iin = -1, alpha_visc = -0.75, hxsch = 3.0, alpha_irad = -0.75, eta = 0.1, rlosch = 3.0, norm = 1,thcent = thcent, thfwhm = thfwhm)


   #convolve with response function for each wavelength
   echolc = mc.mc3(tlc,xlc,taunow,psinow)[idxlo:]
    
  psi_out.append(psinow)  
  fnunow = 1.*echolc
  
  
  
  #resample to simulate uneven observations 
  #if dtmeanin[i] <= 0 then dont resample just use same timegrid
  dtavenow = dtmeanin[i]
  if (dtavenow > 0):
   datin = np.zeros((ntlc,3))
   datin[:,0] = tlc[idxlo:]
   dout = mrs.myresample('',[''],dtavenow,sampmin=sampmin,sampcode=3,datin=datin)
   tout = dout[:,0]
  else:
   tout = tlc[idxlo:]  
  
  tplotlo = np.min(tout)
  
  ngap = len(gap)
  if (ngap != 0):
   ntout = np.shape(tout)[0]
   idexc = []
   idinc_gap = []
   for ig in range(ngap):
    tlonow = gap[ig][0]
    thinow = gap[ig][1]
    #idlo = np.where((tout - tplotlo > tlonow))[0]
    #idhi = np.where((tout - tplotlo < thinow))[0]
    #a = list(np.concatenate((idlo,idhi)))#list(np.where((tout < tlonow) & (tout > thinow))[0])
    a = np.where((tout - tplotlo > tlonow) & (tout - tplotlo < thinow))[0]
    #print a,tlonow, thinow, np.min(tout), np.max(tout),'bug hunt'
    idexc.append(a)
    idinc_gap.append(np.array([j4 for j4 in range(ntout) if j4 not in a]))
   idexc = np.array([j2 for i2 in idexc for j2 in i2]) 
   #idinc = np.arange(ntout)
   #idinc = np.where(idinc
   
   idinc = np.array([j3 for j3 in range(ntout) if j3 not in idexc])
   
   ninc = np.shape(idinc)[0]
   #raw_input()
   #tout_1stgap = tout[idinc_gap[0]]
   tout = tout[idinc]
   
    
  
  #fout_1stgap = np.interp(tout_1stgap,tlc[idxlo:],fnunow)
  fout = np.interp(tout,tlc[idxlo:],fnunow)
  
  
  
  
  #if have blr light curve, snr specifies variability amplitude / errobar
  if (wavin[i] == -1.0):
   fdiskmean = 0
   sdwav = 1.0
   
  if (wavnow == 0.0):
   fdiskmean = mean_x
   sdwav = sd_x
  
  #scale to the correct flux and variability amplitude
  
  #if (onlyuse1stgap_4_scaling == 1):
  # emean = np.mean(fout_1stgap)
  # esd   = np.mean(fout_1stgap)
  # print 'only use 1st gap 4 scaling', emean,esd,sdwav,fdiskmean
  #else:
  emean = np.mean(fout)
  esd   = np.std(fout)
  
  fnunow = (fout - emean)*sdwav/esd + fdiskmean
    
  
  
  #plot during routine
  if (diag_plot == 1):
   fig = plt.figure()
   ax1 = fig.add_subplot(311)
   ax1.plot(tlcout,xlcout)
   ax2 = fig.add_subplot(312)
   ax2.plot(tout,fnunow)
   ax3 = fig.add_subplot(313)
   ax3.plot(taunow,psinow)
   if dirsave is not None:
    plt.savefig(dirsave+'/fig_diagplot_'+np.str(i)+'.pdf')
  
  
  
  #add noise (if have blr light curve, snr specifies variability amplitude / errobar)
  if (sigforce[i] == -1.0):
   #if (wavin[i] == -1 or wavnow == 0.0):
   sig    = np.zeros(np.shape(fnunow)[0])
   sig[:] = sdwav/snr[i]
   #else:
   # sig    = sdwav/snr[i]#fnunow/snr[i]#fnunow/snr[i]
  else:
   sig = np.ones(np.shape(fnunow)[0])*sigforce[i]
  
  
  #set snr based on sd at each interval rather than globally
  if (noise_gap == 1):
   tlonow = 0.0#tplotlo
   sig = np.ones(np.shape(fnunow)[0])  
   for ig in range(ngap):
    if (ig > 0):
     tlonow = gap[ig-1][1] + tplotlo
    #if (ig == 0):
    thinow = gap[ig][0] + tplotlo
    #elif (ig < ngap - 1):
    # thinow = 
    
    #if (ig < ngap - 1):
    # thinow = gap[ig][0] + tplotlo
    
    
    idincnow = np.where((tout > tlonow) & (tout < thinow))[0]
    stdnow   = np.std(fnunow[idincnow])
    meannow  = np.mean(fnunow[idincnow])
    sig[idincnow] = stdnow/snr[i]
    
    
    
    
  
  
  #print 'before adding noise',fnunow[:10],sigforce[i]
  nfnunow = np.shape(fnunow)[0]
  fnunonoise = 1.*fnunow
  fnunow = mr.normdis(nfnunow, fnunow, sig)
  #for i in range(np.shape(fnunow)[0]):
   #print fnunow[i],fnunonoise[i],np.abs(fnunow[i]-fnunonoise[i])/sig[i]
  #print('sd check n.o points outside error bars (should be ~ 0.32 for gaussian noise)', np.shape( np.where( np.abs(fnunow-fnunonoise)/sig > 1)[0] )[0]/(1.*np.shape(fnunow)[0]))
  sigsave.append(sig)
  t_out.append(tout)
  echosave.append(fnunow)#echosave.append(fnunow)
  
  
 
 
 
 
 #plot result to test
 #print 'making diag_plot', diag_plot
 if (diag_plot == 1):
  col = ['k','purple','b','cyan','green','orange','red']*100
  fig = plt.figure()
  ax1 = fig.add_subplot(211)
  ax2 = fig.add_subplot(212)
  for i in range(nwav):
   if (fakeplotnorm == 1):
    em = np.mean(echosave[i])
    esd = np.std(echosave[i])
    ax1.errorbar(t_out[i],(echosave[i] - em)/esd,sigsave[i]/esd,ls='',color=col[i])
   else:
    ax1.errorbar(t_out[i],echosave[i],sigsave[i],ls='',color=col[i])
   
   ax2.plot(taunow,psi_out[i],color=col[i])
   taumean = np.sum(taunow*psi_out[i])/np.sum(psi_out[i])
   ax2.plot([taumean,taumean],[0,1],color=col[i])
  ax2.set_xlim([0,30])
  ax2.tick_params(axis='x',which='minor',bottom='on')
   #print taunow[:10]
   #print psi_out[i][:10]
  if dirsave is not None:
   plt.savefig(dirsave+'/myfake_test.pdf')
 
 
 
 
 #if dirsave not None on then make a folder and save in the format required by cream
 if dirsave is not None:
  f = open('creamnames.dat','w')
  for i in range(nwav):
   wavnow = wav[i]
   tnow = np.array(t_out[i])
   
   
   fnow = np.array(echosave[i])
   signow = np.array(sigsave[i])
   nt = np.shape(tnow)[0]
   
 
   dat = np.zeros((nt,3))
   dat[:,0] = tnow
   dat[:,1] = fnow
   dat[:,2] = signow
   fname = 'synth_'+np.str(int(wavnow))+'.dat'
   #print fname
   os.getcwd()
   #print "'"+fname+"' "+np.str(wavnow)+"\n"     
   np.savetxt(fname,dat)
   f.write("'"+fname+"' "+np.str(wavnow)+"\n")
  f.close()
 
 
  if (tfx == 1):
   f = open('cream_tfx.par','w')
   f.write(np.str(T0v)+' 0.001 \n')
   f.write(np.str(T0x)+' 0.001 \n')
   f.write(np.str(sv)+' 0.001 \n')
   f.write(np.str(sx)+' 0.002 \n')
   f.close()
   os.system('mv cream_tfx.par ./'+dirsave)
   
  if (affine == 1):
   f = open('cream_affine.par','w')
   for ia in paraffine:
    f.write(np.str(ia)+' \n')
   f.write(np.str(naffine)+' \n')
   f.close()
   os.system('mv cream_affine.par ./'+dirsave)
  
  os.system('mv synth* ./'+dirsave)
  os.system('mv creamnames.dat ./'+dirsave)
  

 echo_lightcurves = []
 #print('nwav',nwav)
 #print('sigsave',sigsave)
 #print('end of sigsave')
 for i in range(nwav):
  t = t_out[i]
  x = echosave[i]
  s = sigsave[i]
  echo_lightcurves.append(np.array((t,x,s)).T)

 output = {'driver time axis':tlcout,
           'driver values':xlcout,
           'echo light curves':echo_lightcurves,
           'response function time axis':taunow,
           'response function values':psi_out,
           'accretion rate':emdot}
 return(output)






 







#rsinc ds207@lucky2.st-andrews.ac.uk:/microlens/ds207/fortcode/cream_export/katelc_group_1febmerge_* ./sd_hilag_111117/
import numpy as np
import glob
import matplotlib.pylab as plt
import os
import mymedquart as mm
import myrandom as mr
import myvarlc as mvar

recalc = 1
plotmoddat = 0
contth = 1
#if contth = 1 and detect ncontfail lags (registers as failed) just in case acidentally set too many lags varying and not kept a reference lag
ncontfail = 3
#tplotlim = 
#if idburnin = 0.66 then use only final third of points to calculate uncertainties and parameters
idburnin = 0.66
labsave = ['mdot','cosinc']
axlab   = [r'$\dot{M}$ $(M_\odot^2\mathrm{yr}^{-1})$',r'$\cos i$']


pwd = os.getcwd()

if (recalc == 1):
 dmain = '/microlens/ds207/fortcode/cream_export/cream_python/katelc_group_1febmerge_*'
 #'/Users/ds207/Google Drive/cont_222_thlag/katelc_group_1febmerge_1'#'/Users/ds207/Google Drive/sd_hilag_111117/katelc_group_1febmerge_*'#'/Users/ds207/Google Drive/cont_lag_tophat_241117/katelc_group_1febmerge_*'#'/Users/ds207/Google Drive/sd_hilag_111117/katelc_group_1febmerge_*'#'/Users/ds207/Documents/standrews/sta/lightcurves/sd_hilag_111117/katelc_group_1febmerge_*'
 
 nits = 1000
 
 
 dsubs   = glob.glob(dmain)
 targ    = []
 targ_good = []
 cisqred = []
 snrstatsave = []
 fvm = []
 sigxs = []
 #cisave      = []
 #cisigsave   = []
 parsave     = []
 parsigsave  = []
 
 failsave = []
 #mdotsave    = []
 #mdotsigsave = []
 lagsave     = []
 siglagsave = []
 targsave = []
 cadmed = []
 frac2tot_plot = []
 dtcombmedsave_plot = []
 dtcombmed_sig_plot = []
 lagsaveth = []
 siglagsaveth = []
 snr_kate = []
 
 
 for dirnow in dsubs:
  os.chdir(dirnow)
  
  targnow = glob.glob('rm*')
 
  idrej = []
  ntn = len(targnow)
  for it in range(ntn):
   os.chdir(dirnow)
   tnow = targnow[it]
   
   os.chdir(tnow)
   
   try:
    #figure out how many light curves are at the same wavelength
    wav = []
    with open('creamnames.dat') as f:
     parline = f.readlines()
     parline = [x.strip() for x in parline]
    f.close()
    
 
    nlctemp = len(parline) 
    for i in range(nlctemp):
     pl = parline[i].split()
     wav.append(np.float(pl[-1]))
    
    wavu = np.unique(wav)
    nwavu = np.shape(wavu)[0]
    
    try:
     opdir = sorted(glob.glob('output_2*'))[-1]
    except:
     idrej.append(it)
     continue
    
    
    
    dnow = np.loadtxt(opdir+'/outputpars.dat')
    nitsnow = np.shape(dnow[:,0])[0]
    idit = int(nitsnow*idburnin)
    
    mdotnow = dnow[idit:,2]
    cincnow = dnow[idit:,3]
    
    a = mm.mymedquart(mdotnow,conlev=0.68)
    mdotmed = a[0]
    mdotlo  = a[1]
    mdothi  = a[2]
    
    
    a = mm.mymedquart(cincnow,conlev=0.68)
    cincmed = a[0]
    cinclo  = a[1]
    cinchi  = a[2]
    
    
    
    #if we have line lags load those too
    try:
     tfline = np.loadtxt(opdir+'/outputpars_th.dat')[idit:,:]
     linelag = 1
    except:
     linelag = 0 
    
    
    
    tfmod = np.loadtxt(opdir+'/plots/modeltf.dat')
    tfmod_sig = np.loadtxt(opdir+'/plots/modeltf_sig.dat')
    taumod = tfmod[:,0]
    tfmod = tfmod[:,1:]
    tfmod_sig = tfmod_sig[:,1:]
    ntau = np.shape(taumod)[0]
    
    wavnow = -22.0
    nlc= np.shape(tfmod)[1]
    #save the lags and use monte carlo techniques to retrieve error on lags
    
    lsnow = []
    lssignow = []
    print ''
    print 'target',tnow
    idnon0 = 0
 
 
    print 'hererre contth',contth     
    if (contth == 1):
     print 'a cont'
     thinfo = np.percentile(tfline[:,:nlc],[50,16,84],axis=0)
     #list the indicees of different mean lags (as same filter tied together), and non zero lags 
     print 'a2 cont'
     idu = np.unique(thinfo[0,:],return_index=True)[1]
     idus = np.sort(idu)
     #print 'b cont'
     if (np.shape(idu)[0] == ncontfail):
      print 'failsave',failsave
      failsave.append(tnow+' lag lc label prob')
      continue
     
     #print 'checking lagsave'
     #print thinfo[0,:]
     #print wav
     #print tnow
     #print idu
     #print idus
     #raw_input()
       
     lagsaveth.append(thinfo[0,idus])
     siglagsaveth.append((thinfo[2,idus] - thinfo[1,idus])/2)
     print 'lags',thinfo[0,idu]
     
     #failed if ncontfail lags
  
 
    if (contth ==0):
     for ilc in range(nlc):
      wn = wav[ilc]
      print 'wav wavnow',wn,wavnow
      #skip if shared wavelength
      if (wn == wavnow and ilc != idnon0):
       continue 
      
      #line
      elif (wn == -1):
       if (ilc == 0):
        tfeg = tfline[idit,:]
        idnon0 = np.where(tfeg != 0)[0][0]
       tmline = tfline[idit:,ilc]
       a = np.percentile(tmline,[50,16,84])
       print 'lag',a
      #continuum 
      else:    
       taumean = []
       print 'monte carloing lags...'
       for iteration in range(nits):
        psinew = mr.normdis(ntau,tfmod[:,ilc],tfmod_sig[:,ilc])
        taumean.append( np.sum(taumod*psinew)/np.sum(psinew) )
       tm = np.array(taumean)
       a = mm.mymedquart(tm,conlev=0.68)
  
      
      
      lsnow.append(a[0])
      lssignow.append( (a[2] - a[1])/2 )
      wavnow = wn
     
    
    #if we are stepping continuum together using top hat lags do this now
    
    
    
     if (len(lsnow) > nwavu):
      lsnow = lsnow[:nwavu]
     if (len(lssignow) > nwavu):
      lssignow = lssignow[:nwavu]
  
     lagsave.append(lsnow)
     siglagsave.append(lssignow)  
    
    
    
      
    print 'Done with monte carloing lags'
    
    
    
     
    
    #load model and data and plt
    model = np.loadtxt(opdir+'/plots/modellc.dat')
    ntmod = np.shape(model[:,0])[0]
    
    tmod = model[:,0]
    xmod = model[:,1:]
    
    fdat = sorted(glob.glob(opdir+'/plots/data_echo_dat_*'))
    cisqsum = 0
    nsum    = 0
    snrstatsum = 0
    
    if (plotmoddat == 1):
     fig = plt.figure()
     idpl = np.where(xmod[:,0] > 0.0)[0]
     tplotlim = [tmod[idpl[0]],tmod[idpl[-1]]]
    
    fvar = []
    sigfvar = []
    xsnorm=[]
    frac2tot = []
    #ntot = 0
    #top = 0.d0
    #bot = 0.d0
    timeu = [[] for i in range(nwavu)]
    for ilc in range(nlc):
     wavnow = wav[ilc]
     idu = np.where(wavu == wavnow)[0][0]
     
     dat = np.loadtxt(fdat[ilc])
     ndat = np.shape(dat[:,0])[0]
     tdat   = dat[:,0]
     timeu[idu].append(tdat)
     
     xdat   = dat[:,1]
     sigdat = dat[:,2]
     xm = np.mean(xdat)
     #ntot = ntot + ndat
     
     #calculate fvar
     print 'calculate fvar'
     variance = np.sum((xdat - xm)**2)/(ndat-1)
     sigmean2 = np.sum(sigdat**2)/ndat
     sigxsnow = variance - sigmean2
     fvarnow = sigxsnow/xm**2 #np.sqrt( np.sum((xdat - xm)**2 - sigdat**2 )/ndat ) / xm
     #print 'fvarnow',fvarnow
     frac2totnow = sigxsnow/variance
     frac2tot.append(frac2totnow)     
     print 'here -2'
     fvar.append(fvarnow)
     xsnorm.append(sigxsnow)
     #now calculate kate snr 
     b = fdat[ilc]+' '+ np.str(mvar.linvarfit(tdat,xdat,sigdat)[1])
     snr_kate.append(b+'\n')
     
     #sdm2=np.mean(sigdat**2)
     #print 'sdm2',sdm2,ndat,fvarnow,np.sum((xdat - xm)**2 - sigdat**2)
     #print (np.sqrt(sdm2/ndat) / xm )**2,'asdfad'
     #print (np.sqrt(1/2./ndat)*sdm2/xm**2/fvarnow)**2,'asdasd'
     fvarsignow = (np.sqrt(1/2./ndat)*sigmean2/xm**2/np.sqrt(fvarnow))**2 - (np.sqrt(sigmean2/ndat) / xm )**2
     #sigfvar.append(fvarsignow)
     print 'variance,mean square error',variance,sigmean2
     print 'normxs,xs out',fvarnow,sigxsnow
     print 'Fvar,sig Fvar out',np.sqrt(fvarnow),np.sqrt(fvarsignow)
     print 'frac2tot', frac2totnow
     #print 'i am here',ilc,nlc
     if (plotmoddat == 1):
      ax1 = fig.add_subplot(nlc,1,ilc+1)
      ax1.errorbar(tdat,xdat,sigdat,ls='',color='k',label=None)
      #ax1.plot(tmod,xmod[:,ilc],label=np.str(wav[ilc]))
      ax1.plot(tmod,xmod[:,ilc],label=None)
      xdm = np.mean(xdat)
      xsd = np.std(xdat)
      ax1.set_ylim([xdm-3.5*xsd,xdm+3.5*xsd])
      ax1.set_xlim(tplotlim)
      ax1.set_ylabel(r'flux $'+np.str(int(wav[ilc]))+' \AA$')
      #ax1.legend()
      if (ilc == nlc-1):
       fig.tight_layout()
       plt.savefig('creamplot.pdf')
     
     #print 'i am here',ilc 
     snrstatnow = (np.max(xdat) - np.min(xdat))/np.median(sigdat)
     snrstatsum = snrstatnow + snrstatsum
     xmitp = np.interp(tdat,tmod,xmod[:,ilc])
     cisqsum = cisqsum + np.sum(((xmitp - xdat)/sigdat)**2)
     nsum = nsum + ndat
    
    #print 'here -1'
    frac2tot_plot.append(np.mean(frac2tot))
    #calculate the cadence of each light curve
    dtcombmed_save    = []
    dtcombmedsig_save = []
    #print 'here 0 ',nwavu
    
    for i in range(nwavu):
     tcp = []
     tcp = tcp + [list(timeu[i][it]) for it in range(len(timeu[i]))]
     tcp = np.concatenate(tcp)
     tcomb = np.sort(tcp)
     #print 'here 1'
     dtcomb = tcomb[1:] - tcomb[:-1]
     #print 'here 2'
     tcombmed = np.percentile(dtcomb,[50,16,84])
     ntot = np.shape(tcp)
     tcpmax = np.max(tcp)
     tcpmin = np.min(tcp)
     dtcombmed_sig = (tcombmed[2] - tcombmed[1])/2
     dtcombmed = (tcpmax - tcpmin)/ntot#tcombmed[0]
     dtcombmed_save.append(dtcombmed)
     dtcombmedsig_save.append(dtcombmed_sig)
     #print i,nwavu,'here 3', dtcombmed_save
     #raw_input()
     
    #print 'i am here'
    dtcombmedsave_plot.append(dtcombmed_save)
    dtcombmed_sig_plot.append(dtcombmedsig_save)
    snrstatsave.append(snrstatsum/nlc) 
    parsave.append([mdotmed,cincmed])
    parsigsave.append([(mdothi - mdotlo)/2,(cinchi - cinclo)/2])  
    #mdotsave.append(mdotmed)
    #mdotsigsave.append((mdothi - mdotlo)/2)
    #cisave.append(cincmed)
    #cisigsave.append((cinchi - cinclo)/2)
    cisqred.append(cisqsum/nsum)
    fvarmean = np.mean(fvar)
    sigxsmean = np.mean(xsnorm)
    #print 'fvarmean info start'
    fvm.append(fvarmean)
    sigxs.append(sigxsmean)
    #print 'normxs info',fvm
    #print 'xs info',sigxs
    targ = targ + [targnow[it] for it in range(ntn) if it not in idrej ] 
    targsave.append(tnow)
   except:
    failsave.append(tnow)
 
   print ''
   print ''
 
 os.chdir(pwd)
 
 
 
 #save data 
 f = open('contlags_fail.txt','w')
 for fnow in failsave:
  f.write(fnow+'\n')
 f.close()
 
 parsave_plot = np.array(parsave)
 np.savetxt('parsave_plot.txt',parsave_plot)
 
 parsigsave_plot = np.array(parsigsave)
 np.savetxt('parsigsave_plot.txt',parsigsave_plot)
 
 lagsave_plot = np.array(lagsave)
 np.savetxt('lagsave_plot.txt',lagsave_plot)
 
 lagsigsave_plot = np.array(siglagsave)
 np.savetxt('lagsigsave_plot.txt',lagsigsave_plot)
 
 cisqred_plot = np.array(cisqred)
 np.savetxt('cisqred_plot.txt',cisqred_plot)
 
 snrstatsave_plot = np.array(snrstatsave)
 np.savetxt('snrstatsave_plot.txt',snrstatsave_plot)
 
 frac2tot_plot = np.array(frac2tot_plot)
 np.savetxt('frac2tot_plot.txt',frac2tot_plot)
  
 
 f = open('snr_kate.txt','w')
 for fn in snr_kate:
  f.write(fn)
 f.close()
 
 fvm_plot = np.array(fvm)
 np.savetxt('xsnorm_plot.txt',fvm_plot)

 
 sigxs_plot = np.array(sigxs)
 np.savetxt('xs_plot.txt',sigxs_plot)
 
 
 dtcombmedsave_plot = np.array(dtcombmedsave_plot)
 np.savetxt('cadence_plot.txt',dtcombmedsave_plot)
 
 dtcombmed_sig_plot = np.array(dtcombmed_sig_plot)
 np.savetxt('dtcombmed_sig_plot.txt',dtcombmed_sig_plot)

 
 lagsaveth_plot = np.array(lagsaveth)
 np.savetxt('lagsaveth_plot.txt',lagsaveth_plot)
 
 lagsavesigth_plot = np.array(siglagsaveth)
 np.savetxt('lagsavesigth_plot.txt',lagsavesigth_plot)
 

 
 
#extract numbers from targsave array
ts = [int(t[2]+t[3]+t[4]) for t in targsave]
ts = np.array(ts)

np.savetxt('targsave_op.txt',ts)

#nsave = len(parsave)
#npar = len(parsave[0])
#f = open('contlags_op.dat','w')
#f.write('target cisq/n '+np.str(npar)+' prameters and errors '+np.str(nlc)+' lags and errors (one for each light curve)\n')
#
#for i in range(nsave):
# a = ''
# for ip in range(npar):
#  a = a+np.str(parsave[i][ip])+' '+np.str(parsigsave[i][ip])+' '
# b = ''
# for ip in range(nwavu):
#  b = b+np.str(lagsave[i][ip])+' '+np.str(siglagsave[i][ip])+' '
# 
# f.write(targ[i]+ ' '+np.str(cisqred[i])+' '+ a + b +'\n')
# 

# targw = targw + ' '+targ[i]
# lnow = ''
# lagw = ['']*nlc
# siglagw=['']*nlc
# for il in range(nlc):
#  lagw[il]= lagw[il]+' '+np.str(lagsave[i][il])
#  siglagw[i] = siglagw[i]+ ' '+np.str(siglagsave[i][il])
#
# parw = ['']*npar
# sigparw = ['']*npar
# for ip in range(npar):
#  parw[ip] = parw[ip] + 

#f.close()






#finish this later to be able to load pre calculated lags from earlier up to remake plots rather than recalculate everythong
#with open('contlags_op.dat') as f:
#    content = f.readlines()
#f.close()
#
#ntarg = len(content)
#targ=[]
#cisqred= [] 
#for i in range(ntarg):
# cnow = content[i].split()
# targ.append(cnow[0])
# cisqred.append(
#


#
##make plots
#os.chdir(pwd)
##load data (if just redooing plot can skip to here)
#
#parsave_plot = np.loadtxt('parsave_plot.txt')
#parsigsave_plot = np.loadtxt('parsigsave_plot.txt')
#lagsave_plot = np.loadtxt('lagsave_plot.txt')
#lagsigsave_plot = np.loadtxt('lagsigsave_plot.txt')
#cisqred_plot = np.loadtxt('cisqred_plot.txt')
#snrstatsave_plot = np.loadtxt('snrstatsave_plot.txt')
#
#
#
#
#
#
#nsave,npar = np.shape(parsigsave_plot)
#for ip in range(npar):
# fig = plt.figure()
# ax1 = fig.add_subplot(111)
# 
# ynow = []
# xnow = []
# znow = []
# 
# 
# alphanow = 1/parsigsave_plot[:,ip]
# alphanow = alphanow/np.max(alphanow)
# for it in range(nsave):
#  xnow.append(cisqred_plot[it])
#  ynow.append(parsave_plot[it,ip])
#  znow.append(alphanow[it])
#  ax1.scatter(xnow[it],ynow[it],znow[it],color='b')
# 
# 
# ax1.set_xlabel(r'$\chi^2/n$')
# ax1.set_ylabel(axlab[ip])
# 
# plt.savefig('fig_'+labsave[ip]+'.pdf')
#
#
#
#
#
#
##make plot of snrstat (max(flux) - min(flux) / median errorbar) vs i - g continuum lag
#idlaghi = 1
#idlaglo = 0
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#for it in range(nsave):
# x = lagsave_plot[it,idlaghi] - lagsave_plot[it,idlaglo]
# y = snrstatsave_plot[it]
# sigx = np.sqrt( lagsigsave_plot[it,idlaghi]**2 + lagsigsave_plot[it,idlaglo]**2 )
# ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
# ax1.scatter(lagsave_plot[it,idlaghi] - lagsave_plot[it,idlaglo],snrstatsave_plot[it],color='k')
#
#ylim = list(ax1.get_ylim())
#ax1.plot([0.,0.],ylim,color='k',linewidth=3)
#ax1.set_ylim(ylim)
#ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
#ax1.set_ylabel('SNR = (Max flux - Min flux)/Median Error) \n averaged over all telscopes per filter')
#plt.savefig('fig_snrplot.pdf')
#
#   


#script to merge light curves from two or more telescopes using drw or mean/rms approximation

import numpy as np
import glob
import os
import sys
import re
import myfitrw_current as mf

ifit = 1

#/Users/ds207/Documents/standrews/sta/python/mymerge_test/ps/rm005_gi_cream_notag_ps.dat

#link to the directory containing light curves (light curves will be merged relative to the first entry
dir_data = ['/Users/ds207/Documents/standrews/sta/python/mymerge_test/sdss/',
'/Users/ds207/Documents/standrews/sta/python/mymerge_test/ps/'] 

#search string to load all light curves
data_search = ['*rm*','*rm*']

#light curve file names should have an i.d number indicating to which target they belong.
#script below scans for these and casts them into a 3 digit number (if more than 1000 targets to merge,
#change this to 4 digit)









nmerge = len(dir_data)

#lc = [[] for i in range(nmerge)]
lc  = [None]*nmerge

for i in range(nmerge):
 flist   = glob.glob(dir_data[i]+data_search[i])
 nflist  = len(flist)
 id_list = [map(int, re.findall(r'\d+', master_list[i]))[-1] for i in range(nflist)]#"%03d"%a
 
 if (i == 0):
  master_list = list(flist)
  id_master = list(id_list)
  nflist_master = len(id_master)
 
 #fit data either using mean_rms or using rw fitting or iterated RM RMS_extra fitting
 for idx in range(nflist): 
  dat = np.loadtxt(flist[idx])
  tdat = dat[:,0]
  xdat = dat[:,1]
  sdat = dat[:,2]
  
  if (ifit == 1):#rms_mean method
   stretch = np.std(xdat)
   os      = np.mean(xdat)
  elif (ifit == 2):#fit rw model to light curve before computing mean and rms of model
   mod = mf.fitrw(tdat,xdat,sdat,floin=-1,fhiin=-1,ploton=0,dtresin=0,nits = 1)
   xmod = 
   stretch = np.std(
# nfile = np
 


import numpy as np
from pylab import *



## a function to load an n dimensional array and output a table of data



##open the text file and store the numbers in an array

##function to load n d array from file with name 'name', and number of rows n_row

def file_load(n,name,no_row):
	
	file=open(name, 'r')
	l=[]
	a=file.readlines()

##the file contents are now stored line by line in a list . Need to extract all the numbers from the values from the list and store them in an array


##len(a) is the number of columns, no_row is no of rows
## remember, rows and columns are opposite for python and fortran

##extract all the spaces values from file
	data=np.zeros((len(a), no_row))
	for i in range(len(a)):
		data[i,:]=a[i].split()

	return(data)
	



import numpy as np
from mylcgen import *
from myrandom import *
import matplotlib.pylab as plt
import myresample as mrs
import myredcisq as mrc

twopi = 2*np.pi

#generate test light curve and add noise
datpre      = mylcgen(tlo=0,thi=100,dt=0.03,iseed=34245)
ndat     = np.shape(datpre[:,0])[0]
datmean  = np.std(datpre[:,1])
sig      = np.ones(ndat)/10*datmean
dat = mrs.myresample(dir='',fname=[''],dtave=2.0,sampmin=0.8,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],sig)).T)
ndat = np.shape(dat[:,0])[0]
sig = dat[:,2]
for i in range(ndat):
 dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
y        = dat[:,1] +50
time     = dat[:,0]

tlo = np.min(time)
thi = np.max(time)
dt  = np.mean(time[1:]-time[:-1])

#input some minimum and maximum frequency range for the Fourier terms
flo = 0.5/(thi-tlo)
fhi = 2.0/dt



#angular frequency
w = np.arange(flo,fhi+flo,flo)*2*np.pi
#w = np.arange(0,fhi+flo,5*flo)*2*np.pi
#w[0] = flo/5000
bpl = [0.5,-2,-2]

#prior
if (bpl == []):
 w2 = w**2
else:
 bplin = list(bpl)
 bplin[0] = twopi*bpl[0]
 w2 = (1. + (w/bplin[0])**(bplin[1]-bplin[2])) / (w/bplin[0])**bplin[1]

#Mistake in algebra somewhere means we should square root the prior matrix (not sure why)
w2 = np.sqrt(w2)

#now compute the hessian matrix
nw = np.shape(w)[0]
np2 = 2*nw

cvec = np.ones(np2+1)
hes = np.ones((np2+1,np2+1))
sig2 = sig*sig
#save the sine and cosine value in  2d array
cwt = np.ones((ndat,nw))
swt = np.ones((ndat,nw))
for iw in range(nw):
 cwt[:,iw] = np.cos(w[iw]*time)
 swt[:,iw] = np.sin(w[iw]*time)




#offset
#cos
iwp = 0
hes[0,0] = np.sum(1./sig2)
cvec[0] = np.sum(y/sig2)
for ik in range(1,np2+1,2):
 hes[0,ik] = np.sum(cwt[:,iwp]/sig2)
 hes[0,ik+1] = np.sum(swt[:,iwp]/sig2)
 iwp = iwp + 1

iwp = 0
for ikp in range(1,np2+1,2):
 wp = w[iwp]
 
 wp2 = wp*wp
 cvec[ikp]   = np.sum(y*cwt[:,iwp]/sig2)
 cvec[ikp+1] = np.sum(y*swt[:,iwp]/sig2)
 print iwp,nw
 iw = 0
 
 #offset
 hes[ikp,0] = np.sum(cwt[:,iwp]/sig2)
 hes[ikp+1,0] = np.sum(swt[:,iwp]/sig2)
 
 for ik in range(1,np2+1,2):
  #cosine column
  hnow_c = np.sum(cwt[:,iw]*cwt[:,iwp]/sig2)
  hnow_s = np.sum(swt[:,iw]*cwt[:,iwp]/sig2)
  if (ik == ikp):
   hnow_c = hnow_c + w2[iwp]
  hes[ikp,ik]   = hnow_c
  hes[ikp,ik+1] = hnow_s 
   
  #sine column
  hnow_c = np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
  hnow_s = np.sum(swt[:,iw]*swt[:,iwp]/sig2)
  if (ik == ikp):
   hnow_s = hnow_s + w2[iwp]
  hes[ikp+1,ik]   = hnow_c
  hes[ikp+1,ik+1] = hnow_s 
  iw = iw + 1
  
 iwp = iwp + 1
      
#invert the matrix to get the covariance matrix
cov = np.linalg.inv(hes)


#find the parameters
parm  = cov.dot(cvec)
skout = parm[2::2]
ckout = parm[1::2]
osout = parm[0]




var = np.diag(cov)
sd  = np.sqrt(var)#/1.6
sdos  = sd[0]
sdcos = sd[1::2]
sdsin = sd[2::2]






#!!!!!!!!!!!!!!!!!!!!!!! plot the results
freq = w/2/np.pi
dtres = dt/10
t_ext  = 0.2*(thi - tlo)

tplot = np.arange(tlo-t_ext,thi+t_ext+dtres,dtres)
ntplot = np.shape(tplot)[0]
xplot = []

#resample using errors to get error snake
nits = 100
xplotsave = np.zeros((ntplot,nits))
xplot0_save = np.zeros((ntplot,nits))
pspecsave = np.zeros((nw,nits))


print 'calculating error sake...'
for i in range(nits):
 print i,nits
 cknew = normdis(1,ckout,sdcos)
 sknew = normdis(1,skout,sdsin)
 osnew = normdis(1,osout,sdos)
 #cknew[0] = ckout[0]
 #sknew[0] = skout[0]
 pspecsave[:,i] = sknew*sknew + cknew*cknew
 
 xplot0_save[:,i] = osnew# + cknew[0]*np.cos(w[0]*tplot) + sknew[0]*np.sin(w[0]*tplot)
 for it in range(ntplot):
  tnow = tplot[it]
  xplotsave[it,i] =  osnew + np.sum(cknew*np.cos(w*tnow) + sknew*np.sin(w*tnow))
  
xplot = np.mean(xplotsave,axis=1)
xplot0_sd = np.ones(ntplot)*sdos

xplotsd = np.std(xplotsave,axis=1)# - xplot0_sd


#plot errors for power spectrum also
pspec = ckout**2 + skout**2
pspecsd = np.std(pspecsave,axis=1)

#plot the input and output light curves to see if they match
xplot0 = np.ones(ntplot)*osout#ckout[0]*np.cos(w[0]*tplot) + skout[0]*np.sin(w[0]*tplot)
#for tnow in tplot:
# xplot.append( np.sum(ckout*np.cos(w*tnow) + skout*np.sin(w*tnow)) )

fig = plt.figure()
ax1 = fig.add_subplot(211) 
ax1.errorbar(time,y,sig,ls='',color='k')
ax1.plot(tplot,xplot0,color='b')
ax1.plot(tplot,xplot0-xplot0_sd,color='b')
ax1.plot(tplot,xplot0+xplot0_sd,color='b')



#calculate the reduced chi squared
print 'chisq info', mrc.myredcisq(time,y,sig,tplot,xplot,npar = 0)



#for i in range(nits):
# ax1.plot(tplot,xplotsave[:,i],color='b',alpha=0.06)
ax1.fill_between(tplot,xplot-xplotsd,xplot+xplotsd,alpha=0.2,color='k',label=None)

ax1.plot(tplot,xplot,color='b')
ax1.set_xlabel('Time (days)')
ax1.set_ylabel('flux')


ax2 = fig.add_subplot(212) 

ax2.plot(freq,pspec,ls='',marker='o',color='k')
ax2.fill_between(freq,pspec-pspecsd,pspec+pspecsd,color='k',alpha=0.2)
#ax2.plot(freq,sdsin,ls='',color='g',marker='o')
#ax2.plot(freq,sdcos,ls='',color='b',marker='o')
ax2.set_xscale('log')
ax2.set_yscale('log')
ax2.set_xlabel('Frequency (cyc/day)')
ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
ax2.grid(True)
plt.savefig('test_r_hesfit.pdf')











    


import pyfits
import numpy




def loadsdssspec(infile, redshift=False):
   """
   Load a 1D SDSS spectrum and extract wavelength solution.
   This does not work for DR9 spectra.
   @param infile: input file name
   @type infile: string
   @param redshift: redshift spectrum?
   @type redshift: bool
   """
   dat = pyfits.open(infile)
   if 'COEFF0' in dat[0].header is False or 'COEFF1' in dat[0].header is False:
       raise Exception('No proper header.')
   else:
       co0 = dat[0].header['COEFF0']
       co1 = dat[0].header['COEFF1']
   if dat[0].data is None:
       raise Exception('This is most likely a DR9 spectrum!!!!')
   dim = numpy.shape(dat[0].data)
   if numpy.shape(dim) == (1,):
       n = len(dat[0].data)
       flux = dat[0].data
       err = numpy.zeros_like(flux)
   else:
       n = len(dat[0].data[0])
       flux = dat[0].data[0]
       err = dat[0].data[2]
   wl = 10**(co0 + co1*numpy.arange(n))
   if redshift:
       wl /= 1 + dat[0].header['Z']
   dat.close()
   return {'wl': wl, 'flux': flux, 'error': err, 'z': dat[0].header['Z']}


#My scatter hist 2 allows Multiple 2d parmparm plots to be plotted on the same window
#Useful for methods paper
# need to make better to allow for par file input write this in different subroutine
# Need to upgrade to allow for multiple plots to be shown on same plot
# allow for forcing x and y lims to be same

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import mymedquart
import scipy
import scipy.stats
import myreadlines
import mystats as ms

def myscatterhist_2(xr,y,ialong,ivert,nalong,nvert,col='b',l='',xlab='',ylab='', xlo = 666,xhi = 666,ylo = 666,yhi = 666,nbins = 100, labstyle='bl', translucency = 0.0, lineplot=[0,0,0,0,0,0,0],title=[''],special =[[0],[0]],sigconts=[], truth = [666,666],showconts=[],scatterskip=1):
 
 
# plot log axis if necessary 
 if (special[0][0] == -2):
  x = np.log10(xr)
 else:
  x = xr
 
 #print x.shape,x.min(),x.max(), special
 #raw_input()
 #x = x[:-500:]
 #y = y[:-500:]
 
 paperplot = 1
 #print 'inside function', l
 #raw_input()
 
 nullfmt   = NullFormatter()         # no labels
 lineplot = np.array(lineplot)#plot the comparison with ccf and cream in anotherwindow
 plotlp   = np.mean(lineplot)
 
 # definitions for the axes 
 
 left0 = 0.1
 right0 = 0.9
 bottom0 = 0.1
 top0 = 0.9
 dspace = 0.05
 scatterfrac = 0.5
 dlineplot = 0.04
 
 onesubsf    = 1.-scatterfrac
 width = (1. - left0 - (1.-right0) - (nalong-1)*dspace)/nalong
 height =( 1. - bottom0 - (1.-top0) -(nvert-1)*dspace)/nvert
 
 left   = left0 + ialong*(dspace + width)
 bottom = top0 - (ivert+1)* height - (ivert)*dspace
 
 width_s   = scatterfrac*width
 height_s  = scatterfrac*height 
 left_hy   = left   + width_s
 width_hy  = width - width_s
 
 bottom_hx = bottom + height_s
 height_hy = height - height_s

 

 rect_scatter = [left, bottom, width_s, height_s]
 rect_histx = [left, bottom_hx, width_s, onesubsf*height]
 rect_histy = [left_hy, bottom, onesubsf*width, height_s]
 rect_line  = [left_hy + dlineplot, bottom_hx + dlineplot, onesubsf*width - dlineplot, onesubsf*height - dlineplot]
 
 #print rect_histx
 #print rect_scatter
 #print width, height#,(1. - left0 - right0 - (nalong-1)*dspace),dspace,1. - left0 - right0
 #print bottom,left,ivert,dspace
 #print height,height_s,height_hy
 #raw_input()

 # start with a rectangular Figure
 #plt.figure(1, figsize=(8,8))
 
 axScatter = plt.axes(rect_scatter)
 axHistx = plt.axes(rect_histx)
 axHisty = plt.axes(rect_histy)

 
 # no labels
 #axHistx.xaxis.set_major_formatter(nullfmt)
 #axHisty.yaxis.set_major_formatter(nullfmt)
 
 # if labstyle = 'bl' turn off tick labeling for all but x for lower plots and y for left plots
 axHistx.set_yticklabels(' ')
 axHistx.set_yticks([])
 axHisty.set_yticklabels(' ')
 axHistx.set_xticklabels(' ')
 axHisty.set_xticklabels(' ')
 axHisty.set_xticks([])
 
 if (labstyle == 'bl'):
  if (ivert != nvert - 1):
   axScatter.set_xticklabels('')
  if (ialong > 0):
   axScatter.set_yticklabels('')
   
 
 
 # the scatter plot:
 axScatter.scatter(x[::scatterskip], y[::scatterskip], color = col, s=0.000015)#np.ones(x.shape[0])*1)

 
 
 #plot the true values if necessary
 if (truth[0] != 666):
  axScatter.vlines(truth[0],ymin=ylo,ymax=yhi,color='k')
 if (truth[1] != 666):
  axScatter.hlines(truth[1],xmin=xlo,xmax=xhi,color='k')
 
 
 
 #print 'scatter plotted', xlo, xhi, ylo, yhi, x.mean(), y.mean()
 # add vertical lines to the plot to indicate the mean and sd of the parameter
 #print x
 #raw_input()
 medloupx = mymedquart.mymedquart(x,0.68)
 
 #print medloupx

 lo = medloupx[1]
 up = medloupx[2]
 med = medloupx[0]
 #axScatter.vlines(med,ylo,yhi,color=col,linewidth=3)
 #axScatter.vlines(lo,ylo,yhi,color=col)
 #axScatter.vlines(up,ylo,yhi,color=col)
 
 medloupy = mymedquart.mymedquart(y,0.68)
 lo = medloupy[1]
 up = medloupy[2]
 med = medloupy[0]
 #axScatter.hlines(med,xlo,xhi,color=col,linewidth=3)
 #axScatter.hlines(lo,xlo,xhi,color=col)
 #axScatter.hlines(up,xlo,xhi,color=col)
 
 
 # now determine nice limits by hand:

 if (int(xlo) == 666 & int(xhi) == 666):
  xhi = x.max()
  xlo = x.min()
  
 if (int(ylo) == 666 & int(yhi) == 666):
  yhi = y.max()
  ylo = y.min()
 
 binwidthx = 1.*(xhi - xlo)/nbins
 binwidthy = 1.*(yhi - ylo)/nbins
 

 axScatter.set_xlim( (xlo, xhi) )
 axScatter.set_ylim( (ylo, yhi) )
 axScatter.set_xlabel(xlab)
 axScatter.set_ylabel(ylab)
 
 if (len(special[1]) > 0):
  temp = special[1]
  if (temp[0] == -3):
   axScatter.set_yticks(np.cos(np.pi*np.array(temp[1:])/180))
   axScatter.set_yticklabels(temp[1:])

 if (len(special[0]) > 0):
  temp = special[0]
  if (temp[0] == -2):
   axScatter.set_xticks(temp[1:])
   axScatter.set_xticklabels(temp[1:])


## plot contours if necessary

 nconts = len(sigconts)
 #print 'OOOOAHHHHHHHHHH!!!!!!!',nconts, xlo, xhi, binwidthx,(xhi-xlo)/binwidthx, ylo, yhi, binwidthy,(yhi-ylo)/binwidthy
 if (nconts > 0):
  # fit a KDE to the data
  
  
  ntemp =  len(x)
  #print ntemp,len(y)
  
  xtemp = np.array(x)
  ytemp = np.array(y)
  idx_xnan = np.isnan(xtemp)
  idx_x0= x == 0
  idx_y0= y == 0
  idx_xinf= abs(x) == np.inf
  idx_yinf= abs(y) == np.inf
  idx_ynan = np.isnan(ytemp)
  xtmean = np.median(xtemp)
  ytmean = np.median(ytemp)  
  xtemp[idx_xnan] = xtmean
  xtemp[idx_x0] = xtmean
  ytemp[idx_y0] = ytmean
  ytemp[idx_ynan] = ytmean
  ytemp[idx_yinf] = ytmean
  xtemp[idx_xinf] = xtmean
  x = np.array(xtemp)
  y = np.array(ytemp)
  
  
  

  both = np.array((x,y)).T
  u_x = ms.stat_uncert(x,sig=0.32)
  u_y = ms.stat_uncert(y,sig=0.32)
  
  u_y_2=ms.sigtheta(y,u_y[1])
  
  print 'UNCERT mmdot', '%.2f' % u_x[0], '%.2f' % u_x[1]
  print 'UNCERT inc', '%.2f' % u_y_2[0], '%.2f' % u_y_2[1]
  print ''
  try:
   pdf1=scipy.stats.kde.gaussian_kde(both.T)
  except:
   for i in range(ntemp):
    print x[i],y[i]
    if ( (x[i] == np.nan) or (x[i] == -np.inf) or (y[i] == np.nan) or ( y[i] ==  -np.inf)): 
     print ' It broke!! ', i, x[i], y[i]
     raw_input()
  
  # create a grid over which we can evaluate pdf
  q,w=np.meshgrid(np.arange(xlo,xhi,binwidthx), np.arange(ylo,yhi,binwidthy))
  r1=pdf1([q.flatten(),w.flatten()])
  # sample the pdf and find the value at the 95th percentile
  s1=[]
  for ic in range(nconts):
   s1.append(scipy.stats.scoreatpercentile(pdf1(pdf1.resample(1000)), sigconts[ic]))
   
  # reshape back to 2d
  r1.shape=(q.shape[0],q.shape[1])

  for ic in range(nconts):
   #print 'OOOOAHHHHHHHHHH!!!!!!!', s1[ic],[sigconts[ic]],ic, nconts, xlo, xhi, binwidthx, ylo, yhi, binwidthy
   axScatter.contour(np.arange(xlo,xhi,binwidthx), np.arange(ylo,yhi,binwidthy), r1, [s1[ic]],colors=col)
   #print 'OOOOAHHHHHHHHHH!!!!!!!', [sigconts[ic]],ic, nconts, xlo, xhi, binwidthx, ylo, yhi, binwidthy
## 
 
 #we need a sensible way to define the number of bins. The routine automatically tries to include everything
 #print xlo, xhi, ylo, yhi, binwidthx, binwidthy, nbins
 #raw_input()
 
 #print xlo, xhi, binwidthx, nbins
 #print ylo, yhi, binwidthy, nbins
 #raw_input()
 binsx = np.arange(xlo, xhi + binwidthx, binwidthx)
 binsy = np.arange(ylo, yhi + binwidthy, binwidthy)
 
 #print '10',binsx.shape[0],binsy.shape[0],xlo,xhi,ylo,yhi,binwidthx, binwidthy
 #print x
 #print translucency, binsx, col
 #raw_input()
 try:
  axHistx.hist(x, bins=binsx, color = col, label=l, alpha = translucency,histtype='step')
  #print 'Histogram good x', x.min(), x.max() 
 except: 
  axHistx.hist(x, bins=binsx, color = col, label=l, alpha = 0.3)
  print 'Histogram problem x', x.min(), x.max() 
 #axHistx.legend(l)
 
 
 #print '11'
 try:
  axHisty.hist(y, bins=binsy, orientation='horizontal', color = col, alpha = translucency,histtype ='step')
 except:
  axHisty.hist(y, bins=binsy, orientation='horizontal', color = col, alpha = 0.3)
  print 'Histogram problem y', y.min(), y.max()
  
 
 axHisty.set_xlabel(title)
 axHisty.xaxis.set_label_position('top')
 axHisty.xaxis.labelpad = 20
 
 axHistx.set_xlim( axScatter.get_xlim() )
 axHisty.set_ylim( axScatter.get_ylim() )

 yt = axHistx.get_ylim() 
 axHistx.vlines(truth[0],0,yt[1],color='k')
 yr = axHisty.get_xlim()
 axHisty.hlines(truth[1],0,yr[1],color='k')
 #print 'xlims', xlo,xhi, np.mean(x), np.std(x),x.shape[0]
 #print 'ylims', ylo,yhi, np.mean(y), np.std(y), y.shape[0]
 
 
 
 
 if (plotlp > 0):
  axline  = plt.axes(rect_line)
  #plot the true value
  axline.vlines(lineplot[0],lineplot[5], lineplot[6], color = 'k')
  axline.hlines(lineplot[0],lineplot[5], lineplot[6], color = 'k')
  
  #cram pt with errorbar then ccf with error bar
  axline.errorbar(lineplot[1],lineplot[3],xerr=lineplot[2],yerr=lineplot[4],color = col, linestyle = 'None')
  axline.plot([lineplot[5],lineplot[6]],[lineplot[5],lineplot[6]], color = 'k') #slope of 1
  axline.set_xlim(0,lineplot[5])
  axline.set_ylim(0,lineplot[6])
  axline.set_xticks([lineplot[5],lineplot[6]])
  axline.set_yticks([lineplot[5],lineplot[6]])
  
  #print lineplot

 return axHistx








#new function to plt the multiple correlation plots with input from a text file
#if lab style = 'bl', axis tick labels are placed only on xaxis of lower plots and y-axis of left plots

def myscatterhistfile(filename,colhist,nvert,nalong,parcols,sublabfile = '',xlim=[], ylim=[], labsin=['',''],labstyle='bl', spec=[[0],[0]], nsplit = 1, nperhist=1, sigconts=[], showconts=[],truth = [666,666], scatterskip=1,nbins=100):
 
 #number of datasets per correlatin plot determined by number of entries in colour list i.e ['r'] just 1 or ['r','b'] two etc 
 if (nsplit>1):
  nperhist = 1
 else:
  nperhist = len(colhist)

 if (nperhist > 1 or nsplit > 1):
  translucency = 0.5
 else:
  translucency = 1.0
 
 with file(filename) as f:
  fname = f.read().splitlines()

 nf  = len(fname)

 taumean_ccf=[]
 taumean_cream=[]
 taumean_real=[]
 tit = []
 #determine if we are making the special ccf plots if ccf and scream data are included
 
 lineplot_check = 0
 for i in range(nf):
  a = str.split(fname[i], ',')

  fname[i] = a[0]
  
  if (len(a) > 5):
   taumean_ccf.append((a[5], a[6]))
   taumean_cream.append((a[3], a[4]))
   taumean_real.append(a[1])
   lineplot_check = 1
   
  if (len(a) > 1):
   tit.append(a[1])
  else:
   tit.append('')
 
 if (lineplot_check > 0): 
  taumean_ccf   = np.double(np.array(taumean_ccf))
  taumean_cream = np.double(np.array(taumean_cream))
  taumean_real  = np.double(np.array(taumean_real))
 

  taumeanccf_min = np.min(taumean_ccf[:,0] - taumean_ccf[:,1] )
  taumeanccf_max = np.max(taumean_ccf[:,0] + taumean_ccf[:,1] )

  taumeancream_min = np.min(taumean_cream[:,0]- taumean_cream[:,1] )
  taumeancream_max = np.max(taumean_cream[:,0] + taumean_cream[:,1] )
 
  lineplotmin = 0#np.min((taumeancream_min,taumeanccf_min))

  
  lineplotmax = np.max((taumeancream_max,taumeanccf_max, taumean_real[i]))*1.2
  
   
 nsublab = len(sublabfile)
 if (nsublab > 0):
  with file(sublabfile) as f:
   sublab = f.read().splitlines()  
 


 
 dat=[]
 #!load the data
 for i in range(nf):
  print fname[i]
  try:
   a   = np.loadtxt(fname[i])
  except:
   a   = myreadlines.myreadlines(fname[i])
  ncol = a[0,:].shape[0]
  for i2 in range(ncol): #kde code for confidence curves doesn't like nans. Change these to median of data. Dodgy but no one will know!
   cmed = np.median(a[:,i2])
   nancheck = np.isnan(a[:,i2])
   a[nancheck,i2] = cmed
   
  dat.append(a)
  #print i, fname[i]
  #print a
  #print a.max(), a.min(), np.std(a)
  #raw_input()
  
 plt.figure(1, figsize=(8,8))
 
 idx = 0
 labs = ['','']
 for ia in range(nalong):
  if (ia == 0):
   ylab = labsin[1]
  else:
   ylab = ''
  
  for iv in range(nvert):
   if (iv == nvert-1):
    xlab = labsin[0]
   else:
    xlab = ''
   ivert  = iv 
   ialong = ia
   

   for i2 in range(nperhist):
    icol = i2
    ccfcomp = [0,0,0,0,0,0,0]
    if (lineplot_check > 0):
     ccfcomp[0] = taumean_real[idx]
     ccfcomp[1] = taumean_cream[idx][0]
     ccfcomp[2] = taumean_cream[idx][1]
     ccfcomp[3] = taumean_ccf[idx][0]
     ccfcomp[4] = taumean_ccf[idx][1]
     ccfcomp[5] = lineplotmin
     ccfcomp[6] = lineplotmax    


    for isplit in range(nsplit):
     if (nsplit > 1):
      icol = isplit
     
     #print idx, len(dat), i2, nperhist
     nd = len(dat[idx])
     npersamp = nd /nsplit
     samp = np.arange(isplit*npersamp,(isplit+1)*npersamp) 
     
     if (showconts[isplit] == 0):
      sigcontsnew = []
     else:
      sigcontsnew = list(sigconts)
     if (len(xlim) > 0 ):
      ax1 =  myscatterhist_2(dat[idx][samp,parcols[0]],dat[idx][samp,parcols[1]],ialong,ivert,nalong,nvert,xlo = xlim[0], xhi = xlim[1], ylo = ylim[0], yhi = ylim[1],col = colhist[icol], labstyle = labstyle, translucency = translucency, lineplot = ccfcomp,title = tit[idx], special = spec, sigconts=sigcontsnew, truth = truth,xlab=xlab,ylab=ylab,scatterskip=scatterskip,nbins=nbins)
     else:
      ax1 = myscatterhist_2(dat[idx][samp,parcols[0]],dat[idx][samp,parcols[1]],ialong,ivert,nalong,nvert,col=colhist[icol], labstyle = labstyle, translucency = translucency, lineplot = ccfcomp,title = tit[idx],special=spec,sigconts=sigcontsnew, truth = truth,xlab=xlab,ylab=ylab,scatterskip=scatterskip,nbins=nbins)
    #print dat[idx]
    #print dat[idx].min(), dat[idx].max(), np.mean(dat[idx]), idx
    
    idx = idx + 1
    

 return()
 



#n = 1000
#nf = 8
#nvert = 4
#nalong =2
#
#colhist=['r']#,'b']
#nperhist = 1
#
#dattest = np.zeros((n,2))
#
#
#fnam = 'histmultidat.dat'
#f = open(fnam,'w')
#for i in range(nf):
# 
# fname = 'test_myscatterhist_'+str(i)+'.dat' +' 1.1 1.0 0.5 1.2 0.5'
# f.write(fname+'\n')
# #ax1 = fig.add_subplot(2,2,i+1)
# dattest[:,0] = np.random.randn(n) + 0.1*i
# dattest[:,1] = np.random.randn(n) + 0.1*i
# np.savetxt(fname,dattest)
#
#f.close()
#
#
#
#ccfc = 'ccfcomp.dat' 
#
#a = myscatterhistfile(fnam,colhist,nvert,nalong,[0,1],xlim = [-5,5], ylim=[-5,5])
#
#
#plt.savefig('figcorplot.png')
#
#


#fits random walk model to set of light curves removes white noise contamination
#fits the true intrinsic variability by modelling out white noise 
#Doesn't work properly yet!


#INPUTS:      
#timall      : [timeall[t1[1..nt1],t2[1..n2],...tnlc[1..ntnlc]],
#yall        : [y1[1..nt1],y2[1..n2],...ynlc[1..ntnlc]],
#sigall      : [sig1[1..nt1],sig2[1..n2],...signlc[1..ntnlc]]
#floin,fhiin : the lower and upper frequencies of the random walk
#ploton      : make diagnostic plot of model fit over data
# dtresin    : how hi res do you want the model (cadence days) if -1, just evaluate on same time grid as data
# nits       : get model errors using monte carlo (set hi (1000) if you need the error envelopes around the model fit or 1 to skip if you dont need the error envelopes) 

#OUTPUTS:
#:parmout      : the output Sk and Ck parameters for the model (f(t) = sum_nw S_k sin(wk t) + C_k cos(wk t))
#:freq         : The frequencies at which the model is evaluated 1...nw
#tplot,xplot   : the high resolution model (resolution specified by dtresin argument)
#covout        : covout[:nparameters,:nparameters] the output covariance matrix for the model fit

import numpy as np
from mylcgen import *
import matplotlib.pylab as plt
import myresample as mrs

twopi = 2*np.pi

#if dtresin = -1 then just use model evaluated at data point time 
def fitrw(timeall,yall,sigall,floin=-1,fhiin=-1,ploton=0,dtresin=-1,nits = 1):
 
 parmout = []
 covout = []
 wnout  = []
 sdwn   = []
 
 
 nlc = len(yall)
 ndatall    = []
 dtall   = []
 tmins = []
 tmaxs = []
 for i in range(nlc):
  ndat = np.shape(timeall[i])[0]
  ndatall.append(ndat)
  tmins.append(np.min(timeall[i]))
  tmaxs.append(np.max(timeall[i]))
  dtall.append(np.median(timeall[i][1:] - timeall[i][:-1]))
 
 
 
 
 tlo = np.min(tmins)
 thi = np.max(tmaxs)
 dt  = np.min(dtall)
 
 #input some minimum and maximum frequency range for the Fourier terms
 if (floin < 0):
  flo = 0.5/(thi-tlo)
 else:
  flo = floin
 
 if (fhiin < 0):
  fhi = 0.5/dt
 else:
  fhi = fhiin
 
 
 #angular frequency
 w = np.arange(flo,fhi+flo,flo)*2*np.pi
 #print flo,fhi,fhiin, 0.5/(thi-tlo)
 #w = np.arange(0,fhi+flo,5*flo)*2*np.pi
 #w[0] = flo/5000
 bpl = [0.5,-2,-2]
 
 #prior
 if (bpl == []):
  w2 = w**2
 else:
  bplin = list(bpl)
  bplin[0] = twopi*bpl[0]
  w2 = (1. + (w/bplin[0])**(bplin[1]-bplin[2])) / (w/bplin[0])**bplin[1]
 
 #Mistake in algebra somewhere means we should square root the prior matrix (not sure why)
 w2 = np.sqrt(w2)
 
 #now compute the hessian matrix
 nw = np.shape(w)[0]
 np2 = 2*nw
 
 
 
 #the following has to be donw for as many light curves as you have
 for ilc in range(nlc):
  sig  = sigall[ilc]
  y    = yall[ilc]
  time = timeall[ilc]
  ndat = ndatall[ilc]
 
  npf   = np2 + 1
  nparm = npf+ndat
  cvec = np.ones(nparm)
  hes = np.ones((nparm,nparm))
  sig2 = sig*sig
  #save the sine and cosine value in  2d array
  cwt = np.ones((ndat,nw))
  swt = np.ones((ndat,nw))
  for iw in range(nw):
   cwt[:,iw] = np.cos(w[iw]*time)
   swt[:,iw] = np.sin(w[iw]*time)
  
  
  
  
  #offset
  #cos
  iwp = 0
  hes[0,0] = np.sum(1./sig2)
  cvec[0] = np.sum(y/sig2)
  for ik in range(1,np2+1,2):
   hes[0,ik] = np.sum(cwt[:,iwp]/sig2)
   hes[0,ik+1] = np.sum(swt[:,iwp]/sig2)
   iwp = iwp + 1
  
  iwp = 0
  for ikp in range(1,np2+1,2):
   wp = w[iwp]
   
   wp2 = wp*wp
   cvec[ikp]   = np.sum(y*cwt[:,iwp]/sig2)
   cvec[ikp+1] = np.sum(y*swt[:,iwp]/sig2)
   #print iwp,nw
   iw = 0
   
   #offset
   hes[ikp,0] = np.sum(cwt[:,iwp]/sig2)
   hes[ikp+1,0] = np.sum(swt[:,iwp]/sig2)
   
   for ik in range(1,np2+1,2):
    #cosine column
    hnow_c = np.sum(cwt[:,iw]*cwt[:,iwp]/sig2)
    hnow_s = np.sum(swt[:,iw]*cwt[:,iwp]/sig2)
    if (ik == ikp):
     hnow_c = hnow_c + w2[iwp]
    hes[ikp,ik]   = hnow_c
    hes[ikp,ik+1] = hnow_s 
     
    #sine column
    hnow_c = np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
    hnow_s = np.sum(swt[:,iw]*swt[:,iwp]/sig2)
    if (ik == ikp):
     hnow_s = hnow_s + w2[iwp]
    hes[ikp+1,ik]   = hnow_c
    hes[ikp+1,ik+1] = hnow_s 
    iw = iw + 1
    
   iwp = iwp + 1
        
  
  
  #white noise model to do with noise on data (no covariance between adjacent white noise parameters)
  #!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!#
  idxc = 0
  for i2 in range(npf,nparm,1):
   hes[i2,i2] = 2./sig2[idxc]
   #covariance with cosine parameters
   hes[1:npf:2,i2] = cwt[idxc,:nw]/sig2[idxc]
   #covariance with sine parameters
   hes[2:npf:2,i2] = swt[idxc,:nw]/sig2[idxc]
   #covariance with offset parameter
   hes[0,i2] = 1./sig2[idxc]
   idxc = idxc + 1
   
   
  #correlation vector for white noise parametere 
  cvec [npf:] = y/sig2
  
  
  
  #!!!!!!!!!!!!#End of white noise bit#!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!#
  #!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!#
  #!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!#
  #!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!##!!!!!!!!!!!!#
  
  
   
  #invert the matrix to get the covariance matrix
  cov = np.linalg.inv(hes)
  
  
  #find the parameters
  parm  = cov.dot(cvec)
  skout = parm[2:npf:2]
  ckout = parm[1:npf:2]
  osout = parm[0]
  whitenoise = parm[npf:]
  
  
  
  
  var = np.diag(cov)
  sd  = np.sqrt(var)#/1.6
  sdos  = sd[0]
  sdcos = sd[1:npf:2]
  sdsin = sd[2:npf:2]
  sdwhitenoise = sd[npf:]
  
  sdwn.append(sdwhitenoise)
  wnout.append(whitenoise)
  parmout.append(parm)
  covout.append(cov)

  freq = w/2/np.pi


  t_ext  = 0.0#0.2*(thi - tlo)

  if (dtresin == 0):
   dtres = dt/10
   tplot = np.arange(tlo-t_ext,thi+t_ext+dtres,dtres)
  elif (dtresin == -1):
   tplot = 1.*timeall[ilc]
  elif (isinstance(dtresin, np.ndarray) == 1):
   tplot = 1.*dtresin
   dtres = tplot[1] - tplot[0]
  else:
   dtres = dtresin
   tplot = np.arange(tlo-t_ext,thi+t_ext+dtres,dtres)
   
  
  
  ntplot = np.shape(tplot)[0]
  xplot = 0*tplot
  #if (ploton == 1):
  #!!!!!!!!!!!!!!!!!!!!!!! plot the results

  
  xplot = []
  
  #resample using errors to get error snake
  #nits = 10
  xplotsave = np.zeros((ntplot,nits))
  xplot0_save = np.zeros((ntplot,nits))
  pspecsave = np.zeros((nw,nits))
  
  
  print 'calculating error sake...'
  for i in range(nits):
   print i,nits
   #cknew = normdis(1,ckout,sdcos)
   #sknew = normdis(1,skout,sdsin)
   #osnew = normdis(1,osout,sdos)
   #need to use multivariate gaussian to sample correlated probability space
   if (nits == 1):
    parmnew = 1.*parm
   else:
    parmnew = np.random.multivariate_normal(parm,cov)
   osnew = parmnew[0]
   cknew = parmnew[1:npf:2]
   sknew = parmnew[2:npf:2]
   whnnew = parmnew[npf:]
   
   #cknew[0] = ckout[0]
   #sknew[0] = skout[0]
   pspecsave[:,i] = sknew*sknew + cknew*cknew
   
   xplot0_save[:,i] = osnew# + cknew[0]*np.cos(w[0]*tplot) + sknew[0]*np.sin(w[0]*tplot)
   for it in range(ntplot):
    tnow = tplot[it]
    
    xplotsave[it,i] =  osnew + np.sum(cknew*np.cos(w*tnow) + sknew*np.sin(w*tnow)) 
   
   if (dtresin == -1):
    xplotsave[:,i] = xplotsave[:,i] + whnnew
  
    
    
  
  xplot = np.mean(xplotsave,axis=1)
  xplot0_sd = np.ones(ntplot)*sdos
  
  xplotsd = np.std(xplotsave,axis=1)# - xplot0_sd
  
  
  #plot errors for power spectrum also
  pspec = ckout**2 + skout**2
  pspecsd = np.std(pspecsave,axis=1)
  
  #plot the input and output light curves to see if they match
  xplot0 = np.ones(ntplot)*osout#ckout[0]*np.cos(w[0]*tplot) + skout[0]*np.sin(w[0]*tplot)
  #for tnow in tplot:
  # xplot.append( np.sum(ckout*np.cos(w*tnow) + skout*np.sin(w*tnow)) )
  if (ploton == 1):
   fig = plt.figure()
   ax1 = fig.add_subplot(211) 
   ax1.errorbar(time,y,sig,ls='',color='k')
   ax1.plot(tplot,xplot0,color='b')
   ax1.plot(tplot,xplot0-xplot0_sd,color='b')
   ax1.plot(tplot,xplot0+xplot0_sd,color='b')
   
   
   
   #calculate the reduced chi squared
   #print 'chisq info', mrc.myredcisq(time,y,sig,tplot,xplot,npar = 0)
   
   
   
   #for i in range(nits):
   # ax1.plot(tplot,xplotsave[:,i],color='b',alpha=0.06)
   ax1.fill_between(tplot,xplot-xplotsd,xplot+xplotsd,alpha=0.2,color='k',label=None)
   
   ax1.plot(tplot,xplot,color='b')
   ax1.set_xlabel('Time (days)')
   ax1.set_ylabel('flux')
   
   
   ax2 = fig.add_subplot(212) 
   
   ax2.plot(freq,pspec,ls='',marker='o',color='k')
   ax2.fill_between(freq,pspec-pspecsd,pspec+pspecsd,color='k',alpha=0.2)
   #ax2.plot(freq,sdsin,ls='',color='g',marker='o')
   #ax2.plot(freq,sdcos,ls='',color='b',marker='o')
   ax2.set_xscale('log')
   ax2.set_yscale('log')
   ax2.set_xlabel('Frequency (cyc/day)')
   ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
   ax2.grid(True)
   plt.savefig('test_r_hesfit_'+np.str(ilc)+'.pdf')
 
  
 return(parmout,covout,freq,tplot,xplot)














timeall = []
sigall  = []
yall    = []

nlc = 1



for i in range(nlc):
 #generate test light curve and add noise
 datpre      = mylcgen(tlo=0,thi=100,dt=0.03,iseed=132423)
 npre     = np.shape(datpre[:,0])[0]
 datmean  = np.std(datpre[:,1])
 snow =  np.ones(npre)/10*datmean 
 dat = mrs.myresample(dir='',fname=[''],dtave=2.0,sampmin=0.8,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],snow)).T)
 ndat = np.shape(dat[:,0])[0]
 sig = dat[:,2]
 for i in range(ndat):
  dat[i,1] = np.random.randn(1)*sig[i] + dat[i,1]#normdis(1,dat[i,1],sig[i])[0]
 sigall.append( sig )
 yall.append( dat[:,1] +50 )
 timeall.append( dat[:,0] )
 
 
#dtres in adds white noise onto model and fits well but... 
a = fitrw(timeall,yall,sigall,ploton=1,dtresin = -1)  
#... if i plot the pure model I get much worse fit to data than expected. 
#... almost like model thinks there is more noise than there is
a = fitrw(timeall,yall,sigall,ploton=1,dtresin = 0.1) 
 


#
#    












# convert time,data,error from mjd,mjd-5000,tohjd
import numpy as np
import os


########################definitions
def hjd2mjd(x):
    x=x-2400000.5
    return(x)
     
def mjd2hjd(x):
    x=x+2400000.5
    return(x)
 
########################

#inps the directory of light curves dir//diragn, fname input file name, sring_func what to do to the times

def mydateconv(dir,diragn,fname,string_func):
 #dir='./5548data/gisella_18nov/'#'../fort/fortcode/mcmcmulti3/'
 #diragn=''#'5548ltonly_july28/'
 pwd=os.getcwd()
 
 #a=raw_input('Enter dir path here if you want to change from :- '+str(dir))
 #if (a != ''):
 #    dir=a
 
 #a=raw_input('Enter agn path here if you want to change from :- '+str(diragn))
 #if (a != ''):
 #    diragn=a
 
 #fname=raw_input('Enter file name here :-')
 fnameop='converted_'+fname
 #a=raw_input('Enter new file name here if you dont like :-'+fnameop)
 #if (a != ''):
 #    fnameop=a
 
 os.chdir(str(dir)+str(diragn))
 
 #print 'write 1 of the following:'
 #print '    hm_num : convert HJD to MJD +num'
 #print '    mh_num : convert MJD to HJD +num'
 #print ':-'
 #a=raw_input()
 a = string_func
 numadd=int(a[a.index('_')+1:])
 a=a[:2]
 

 
 
 dat=np.loadtxt(fname)
 
 ## check the data is in ascending order by date
 idxsort=np.argsort(dat[:,0])
 dat[:,:]=dat[idxsort,:]
 ##
 
 ## Act on input instructions
 if (a=='hm'):
     dat[:,0]=hjd2mjd(dat[:,0])+numadd
 elif (a=='mh'):
     dat[:,0]=mjd2hjd(dat[:,0])+numadd
 else:
     print 'I know not of this jibber jabber you speak, try again.'
 
 
 np.savetxt(fnameop,dat)
 
 os.chdir(pwd)


import numpy as np
import astropy


#function to perform running sigma clip on inpu light curve

def myrunsigclip(t,x,width,sd=4,iters=None):

 xchop = np.zeros((0))
 tchop = np.zeros((0))
 tmin  = t.min()
 tmax  = t.max()
 
 tlo = 1.*tmin
 thi = tlo + width
 tloop=[tlo]
 while (thi <= tmax):
  idx = (t > tlo) & (t <= thi)
  tnow = t[idx]
  xnow = x[idx]
  
  xnow_clip = astropy.stats.funcs.sigma_clip(xnow, sig=sd, iters=iters)
  idxchop = xnow_clip[1]
  xnow_clip = xnow_clip[0]

  
  tchop=np.append(tchop,tnow[idxchop])
  xchop=np.append(xchop,xnow_clip)
  tlo = tlo + width
  thi = thi + width
  tloop.append(tlo)
 return(tchop,xchop,tloop)
  
  
#function to perform running sigma clip on input light curve (no segments re-evauate for each point)

def myrunsigclip2(t,x,width,sd=4,iters=None):
 fracl=[]
 nx = x.shape[0]
 xtally = np.zeros(nx)
 xpicked = np.zeros(nx)
 idx = np.arange(nx)

 xchop = np.zeros((0))
 tchop = np.zeros((0))
 tmin  = t.min()
 tmax  = t.max()
 
 ic = 0
 for tn in t:
 
  tlo = max(tmin,tn-width/2)
  thi = min(tmax,tn+width/2)

  tloop=[tlo]
  #idxnow = idx[(t > tlo) & (t <= thi)]
  idxnow = idx[(t > tlo) & (t <= thi) & (t != tn)]
  xpicked[idxnow] = xpicked[idxnow] + 1#[xa + 1 for xa in xpicked[idxnow]]
  tnow = t[idxnow]
  xnow = x[idxnow]
  
  xnow_clip = astropy.stats.funcs.sigma_clip(xnow, sig=sd, iters=iters)
  idxchop = idxnow[xnow_clip[1]]#xnow_clip[1]
  idxn    = idxnow[np.where(tnow == tn)[0]]
  xnow_clip = xnow_clip[0]
  xtally[idxchop]= xtally[idxchop] + 1
  #xtally[idxn] = xtally[idxn] - 1 #do not count the current value as a success
  
  #tests
  #nxn = np.shape(idxnow)[0]
  #nxc = np.shape(idxchop)[0]
  #print idxnow, nxn
  #print idxchop,nxc
  #print xtally[idxnow],idxn
  #raw_input()
  
  #for i in range(nxn):
  # print i, idxnow
  
  ic = ic + 1
  frac = int(100.*ic/nx)
  if (np.mod(frac,10) == 0):
   test = frac in fracl
   if (test == 0):
    print 'myrunsigclip2',frac,'pc complete'
    fracl.append(frac)

   
 #idxreturn = idx[np.where(np.array(xtally > 0))[0]]
 idxreturn = idx[np.where(np.array(xtally == xpicked))[0]]
 tchop =  t[idxreturn]
 xchop = x[idxreturn]
 
 
 return(tchop,xchop,xpicked,xtally)
 

 


import numpy as np




#combine signals with specific uncertainties and weights r
#should all be on same time axis prior to inserting in here
#
#signal is list of arrays each array is N X 3 with 1,2 and 3 columns are the time, flux 
#and uncertainty axis
#r2 are a list of weights from 0 to 1 on how much to regard each series 
#can use correlation coefficients for this
#first axis indicates time, 2nd indicates separate signals

def comb_signal(x,noise,r):

 n2 = noise**2

 top = np.sum(r*x/n2,axis=1)
 bot = np.sum(r/n2,axis=1)
 signal_out   = top/bot
 signal_noise = np.sqrt(1./bot)
 
 
 #will crash if zeros used for uncertainties
 if 0 in noise:
  raise Exception('cannot have 0 value for noise array in vortexa_combine_signal.py')
  
 
 #nx,ny = np.shape(x)
 #for i in range(nx):
 # print(i,noise[i,:])
 #print('mean',np.mean(x,axis=1))
 #print('rms',np.std(x,axis=1))
 #print('top',top)
 #print('bot',bot)  
 #input()

 return(signal_out,signal_noise)
 
 
  
 
 
 
 
 
 
 
 
 
 
 
 
 
##test combine signal code\
#import mylcgen as mlc
#import matplotlib.pylab as plt
# 
# 
# 
# 
#anoise = 0.1
#bnoise = 0.1
#tlo = 0.0
#thi = 100.0
#dt = 1.0
#
#a = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=tlo,thi=thi,dt=dt,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
#na = np.shape(a[:,0])[0]
#asd = np.std(a[:,1])
#a[:,1] = np.random.randn(na)*anoise*asd + a[:,1]
#
#
#b = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=tlo,thi=thi,dt=dt,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
#nb = np.shape(a[:,0])[0]
#bsd = np.std(a[:,1])
#b[:,1] = np.random.randn(nb)*bnoise*bsd + b[:,1]
#
#
#
#
#
#noise = np.zeros((na,2))
#noise[:,0] = anoise*asd
#noise[:,1] = bnoise*bsd
#
#signal = np.zeros((na,2))
#signal[:,0] = a[:,1]
#signal[:,1] = b[:,1]
#
#
#r  = [1.0,1.0]
#cb = comb_signal(signal,noise,r)
#
#
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.fill_between(a[:,0],signal[:,0]-noise[:,0],signal[:,0]+noise[:,0],label='signal 1',alpha=0.6)
#ax1.fill_between(a[:,0],signal[:,1]-noise[:,1],signal[:,1]+noise[:,1],label='signal 2',alpha=0.6)
#ax1.fill_between(a[-50:,0],cb[0][-50:]-cb[1][-50:],cb[0][-50:]+cb[1][-50:],label='combined signal',alpha=0.6)
#ax1.legend()
#plt.show()
#
#









#sort telescope ids that may not be in order into order
# sort list of integers that include gaps and missing number where 1 number may be huge

import numpy as np



def mytelid_sort(telid):

 a  = list(telid)#[1,2,17,8,1,3,17,5,5,6,8,0,6]
 idasort = np.argsort(a)
 ida_sort_sort = np.argsort(idasort)
 asort = np.array(a)[idasort]
 
 b  = np.arange(np.min(asort),np.max(asort),1)
 c  = np.array(list( set(list(asort)) ^ set( list(b) ) ))
 anew = []
 
 nasort = len(asort)
 
 for i in range(nasort):
  
  anow = asort[i]
  idxless = list(np.where(c < anow)[0])
  idr = 0
 
  if (i < nasort-1):
   anext = asort[i+1]
  if ((anow != 0) & (len(idxless) > 0)):
   
   idxless = list(np.where(c < anow)[0])
   
   idpop = idxless[0]
   
   if (anow != anext):
    d = c[idpop]
    c = np.delete(c,idpop)
    c = np.insert(c,[0],anow)
    c = np.sort(c)
   else:
    d = c[idpop]
   
   anow = d
  

  anew.append(anow)
 
 anew = np.array(anew)[ida_sort_sort]
 
 return(anew)


#My scatter hist 2 allows Multiple 2d parmparm plots to be plotted on the same window
#Useful for methods paper
# need to make better to allow for par file input write this in different subroutine
# Need to upgrade to allow for multiple plots to be shown on same plot
# allow for forcing x and y lims to be same

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import mymedquart
import scipy
import scipy.stats
import myreadlines
import mystats as ms
import matplotlib

def myscatterhist_2(xt,yt,ialong,ivert,nalong,nvert,col='b',l='',xlab='',ylab='', xlo = 666,xhi = 666,ylo = 666,yhi = 666,nbins = 100, labstyle='bl', translucency = 0.0, lineplot=[0,0,0,0,0,0,0],title=[''],special =[[0],[0]],sigconts=[], truth = [666,666],showconts=[],xtick_in_sub=[],ytick_in_sub=[],xtick_in_lab=[],ytick_in_lab=[],scatterskip=1,normto1=1,normed=0,fixcheat = [0.,0.],fss=22):
 nn = np.shape(xt)
 xr = np.zeros(nn)
 yr = np.zeros(nn)
 x  = np.zeros(nn)
 y  = np.zeros(nn)
 
 xr[:]=1.*xt[:]
 yr[:] = 1.*yt[:]
 
# plot log axis if necessary 
 if (special[0] == -2):
  xr = np.log10(xt)
 else:
  xr = 1.*xt

#fix cheat
 x[:] = xr[:] - fixcheat[0]
 y[:] = yr[:] - fixcheat[1] 


 matplotlib.rcParams.update({'font.size': fss})


 #print x.shape,x.min(),x.max(), special
 #raw_input()
 #x = x[:-500:]
 #y = y[:-500:]
 
 paperplot = 1
 #print 'inside function', l
 #raw_input()
 
 nullfmt   = NullFormatter()         # no labels
 lineplot = np.array(lineplot)#plot the comparison with ccf and cream in anotherwindow
 plotlp   = np.mean(lineplot)
 
 # definitions for the axes 
 
 left0 = 0.1
 right0 = 0.87
 bottom0 = 0.1
 top0 = 0.9
 dspace = 0.05
 scatterfrac = 0.6
 dlineplot = 0.04
 
 onesubsf    = 1.-scatterfrac
 width = (1. - left0 - (1.-right0) - (nalong-1)*dspace)/nalong
 height =( 1. - bottom0 - (1.-top0) -(nvert-1)*dspace)/nvert
 
 left   = left0 + ialong*(dspace + width)
 bottom = top0 - (ivert+1)* height - (ivert)*dspace
 
 width_s   = scatterfrac*width
 height_s  = scatterfrac*height 
 left_hy   = left   + width_s
 width_hy  = width - width_s
 
 bottom_hx = bottom + height_s
 height_hy = height - height_s

 

 rect_scatter = [left, bottom, width_s, height_s]
 rect_histx = [left, bottom_hx, width_s, onesubsf*height]
 rect_histy = [left_hy, bottom, onesubsf*width, height_s]
 rect_line  = [left_hy + dlineplot, bottom_hx + dlineplot, onesubsf*width - dlineplot, onesubsf*height - dlineplot]
 
 #print rect_histx
 #print rect_scatter
 #print width, height#,(1. - left0 - right0 - (nalong-1)*dspace),dspace,1. - left0 - right0
 #print bottom,left,ivert,dspace
 #print height,height_s,height_hy
 #raw_input()

 # start with a rectangular Figure
 #plt.figure(1, figsize=(8,8))
 
 axScatter = plt.axes(rect_scatter)
 axHistx = plt.axes(rect_histx)
 axHisty = plt.axes(rect_histy)

 
 # no labels
 #axHistx.xaxis.set_major_formatter(nullfmt)
 #axHisty.yaxis.set_major_formatter(nullfmt)
 
 # if labstyle = 'bl' turn off tick labeling for all but x for lower plots and y for left plots
 axHistx.set_yticklabels(' ')
 axHistx.set_yticks([])
 axHisty.set_yticklabels(' ')
 axHistx.set_xticklabels(' ')
 axHisty.set_xticklabels(' ')
 axHisty.set_xticks([])
 
 if (labstyle == 'bl'):
  if (ivert != nvert - 1):
   axScatter.set_xticklabels('')
  if (ialong > 0):
   axScatter.set_yticklabels('')
   
 
 
 # the scatter plot:
 axScatter.scatter(x[::scatterskip], y[::scatterskip], color = col, s=0.000015)#np.ones(x.shape[0])*1)

 
 
 #plot the true values if necessary
 if (truth[0] != 666):
  axScatter.vlines(truth[0],ymin=ylo,ymax=yhi,color='k')
 if (truth[1] != 666):
  axScatter.hlines(truth[1],xmin=xlo,xmax=xhi,color='k')
 
 
 
 #print 'scatter plotted', xlo, xhi, ylo, yhi, x.mean(), y.mean()
 # add vertical lines to the plot to indicate the mean and sd of the parameter
 #print x
 #raw_input()
 medloupx = mymedquart.mymedquart(x,0.68)
 
 #print medloupx

 lo = medloupx[1]
 up = medloupx[2]
 med = medloupx[0]
 #axScatter.vlines(med,ylo,yhi,color=col,linewidth=3)
 #axScatter.vlines(lo,ylo,yhi,color=col)
 #axScatter.vlines(up,ylo,yhi,color=col)
 
 medloupy = mymedquart.mymedquart(y,0.68)
 lo = medloupy[1]
 up = medloupy[2]
 med = medloupy[0]
 #axScatter.hlines(med,xlo,xhi,color=col,linewidth=3)
 #axScatter.hlines(lo,xlo,xhi,color=col)
 #axScatter.hlines(up,xlo,xhi,color=col)
 
 
 # now determine nice limits by hand:

 if (int(xlo) == 666 & int(xhi) == 666):
  xhi = x.max()
  xlo = x.min()
  
 if (int(ylo) == 666 & int(yhi) == 666):
  yhi = y.max()
  ylo = y.min()
 
 binwidthx = 1.*(xhi - xlo)/nbins
 binwidthy = 1.*(yhi - ylo)/nbins
 

 axScatter.set_xlim( (xlo, xhi) )
 axScatter.set_ylim( (ylo, yhi) )
 axScatter.set_xlabel(xlab)
 axScatter.set_ylabel(ylab)
 
# if (len(special[1]) > 0):
#  temp = special[1]
#  if (temp[0] == -3):
#   axScatter.set_yticks(np.cos(np.pi*np.array(temp[1:])/180))
#   axScatter.set_yticklabels(temp[1:])
#
# if (len(special[0]) > 0):
#  temp = special[0]
#  if (temp[0] == -2):
#   axScatter.set_xticks(temp[1:])
#   axScatter.set_xticklabels(temp[1:])


## plot contours if necessary

 nconts = len(sigconts)
 #print 'OOOOAHHHHHHHHHH!!!!!!!',nconts, xlo, xhi, binwidthx,(xhi-xlo)/binwidthx, ylo, yhi, binwidthy,(yhi-ylo)/binwidthy
 if (nconts > 0):
  # fit a KDE to the data
  
  
  ntemp =  len(x)
  #print ntemp,len(y)
  
  xtemp = np.array(x)
  ytemp = np.array(y)
  idx_xnan = np.isnan(xtemp)
  idx_x0= x == 0
  idx_y0= y == 0
  idx_xinf= abs(x) == np.inf
  idx_yinf= abs(y) == np.inf
  idx_ynan = np.isnan(ytemp)
  xtmean = np.median(xtemp)
  ytmean = np.median(ytemp)  
  xtemp[idx_xnan] = xtmean
  xtemp[idx_x0] = xtmean
  ytemp[idx_y0] = ytmean
  ytemp[idx_ynan] = ytmean
  ytemp[idx_yinf] = ytmean
  xtemp[idx_xinf] = xtmean
  x = np.array(xtemp)
  y = np.array(ytemp)
  
  
  

  both = np.array((x,y)).T
  u_x = ms.stat_uncert(x,sig=0.32)
  u_y = ms.stat_uncert(y,sig=0.32)
  
  u_y_2=ms.sigtheta(y,u_y[1])
  
  print 'UNCERT mmdot', '%.2f' % u_x[0], '%.2f' % u_x[1]
  print 'UNCERT inc', '%.2f' % u_y_2[0], '%.2f' % u_y_2[1]
  print ''
  try:
   pdf1=scipy.stats.kde.gaussian_kde(both.T)
  except:
   for i in range(ntemp):
    print x[i],y[i]
    if ( (x[i] == np.nan) or (x[i] == -np.inf) or (y[i] == np.nan) or ( y[i] ==  -np.inf)): 
     print ' It broke!! ', i, x[i], y[i]
     raw_input()
  
  # create a grid over which we can evaluate pdf
  q,w=np.meshgrid(np.arange(xlo,xhi,binwidthx), np.arange(ylo,yhi,binwidthy))
  r1=pdf1([q.flatten(),w.flatten()])
  # sample the pdf and find the value at the 95th percentile
  s1=[]
  for ic in range(nconts):
   s1.append(scipy.stats.scoreatpercentile(pdf1(pdf1.resample(1000)), sigconts[ic]))
   
  # reshape back to 2d
  r1.shape=(q.shape[0],q.shape[1])

  for ic in range(nconts):
   #print 'OOOOAHHHHHHHHHH!!!!!!!', s1[ic],[sigconts[ic]],ic, nconts, xlo, xhi, binwidthx, ylo, yhi, binwidthy
   axScatter.contour(np.arange(xlo,xhi,binwidthx), np.arange(ylo,yhi,binwidthy), r1, [s1[ic]],colors=col,linewidth=2.0)
   #print 'OOOOAHHHHHHHHHH!!!!!!!', [sigconts[ic]],ic, nconts, xlo, xhi, binwidthx, ylo, yhi, binwidthy
## 
 
 #we need a sensible way to define the number of bins. The routine automatically tries to include everything
 #print xlo, xhi, ylo, yhi, binwidthx, binwidthy, nbins
 #raw_input()
 
 #print xlo, xhi, binwidthx, nbins
 #print ylo, yhi, binwidthy, nbins
 #raw_input()
 binsx = np.arange(xlo, xhi + binwidthx, binwidthx)
 binsy = np.arange(ylo, yhi + binwidthy, binwidthy)
 


 #print '10',binsx.shape[0],binsy.shape[0],xlo,xhi,ylo,yhi,binwidthx, binwidthy
 #print x
 #print translucency, binsx, col
 #raw_input()
 nx = np.shape(x)
 wx = np.ones(nx)
 ny = np.shape(y)
 wy = np.ones(ny)
 
 if (normto1 == 1):
  renorm_x = np.histogram(x,bins=binsx)
  xnmax = np.max(renorm_x[0])
  wx = wx/xnmax
  renorm_y = np.histogram(y,bins=binsy)
  ynmax = np.max(renorm_y[0])
  wy = wy/ynmax
 else:
  wx[:] = 1
  wy[:]= 1 

 
 try:
  print 'weights x',wx[0],xnmax
  ahx = axHistx.hist(x, bins=binsx, weights = wx, color = col, label=l, alpha = 1.0,histtype='step',normed=normed,linewidth=2)
  #print 'Histogram good x', x.min(), x.max() 
 except: 
  try:
   ahx = axHistx.hist(x, bins=binsx, weights = wx, color = col, label=l, alpha = 1.0,normed=1,linewidth=2,histtype='step')
  except:
   ahx = axHistx.hist(x, bins=binsx, weights = wx, color = col, label=l, alpha = 0.3,normed=1,linewidth=2)
   print 'Histogram problem x', x.min(), x.max() 
 #axHistx.legend(l)
 
 
 #print '11'
 try:
  print 'weights y', wy[0],ynmax
  ahy = axHisty.hist(y, bins=binsy, weights=wy, orientation='horizontal', color = col, alpha = 1.0,histtype ='step', normed=normed,linewidth=2)
 except:
  try:
   ahy = axHisty.hist(y, bins=binsy, weights=wy, orientation='horizontal', color = col, alpha = 1.0, normed=1,linewidth=2,histtype='step')
  except:
   ahy = axHisty.hist(y, bins=binsy, weights=wy, orientation='horizontal', color = col, alpha = 0.3, normed=0,linewidth=2)
   print 'Histogram problem y', y.min(), y.max()
 
 

 axHisty.set_xlabel(title)
 axHisty.xaxis.set_label_position('top')
 axHisty.xaxis.labelpad = 20
 
 axHistx.set_xlim( axScatter.get_xlim() )
 axHisty.set_ylim( axScatter.get_ylim() )

 yt = axHistx.get_ylim() 
 axHistx.vlines(truth[0],0,yt[1],color='k')
 yr = axHisty.get_xlim()
 axHisty.hlines(truth[1],0,yr[1],color='k')
 xha = axHistx.get_ylim()
 yha = axHisty.get_ylim()
 print 'yt',yt,'yr',yr,'xha',xha,'yha',yha,'fsdfsdfdsfsfsfsfarghhhhhh',np.max(y), np.max(x)
 axHistx.set_ylim(0.0,yt[1]*1.1)
 axHistx.set_xlim(xlo,xhi)
 
 axHisty.set_xlim(0.0,yr[1]*1.1)
 axHisty.set_ylim(ylo,yhi)
 

 #print 'xlims', xlo,xhi, np.mean(x), np.std(x),x.shape[0]
 
 
 
 if (plotlp > 0):
  axline  = plt.axes(rect_line)
  #plot the true value
  axline.vlines(lineplot[0],lineplot[5], lineplot[6], color = 'k')
  axline.hlines(lineplot[0],lineplot[5], lineplot[6], color = 'k')
  
  #cram pt with errorbar then ccf with error bar
  axline.errorbar(lineplot[1],lineplot[3],xerr=lineplot[2],yerr=lineplot[4],color = col, linestyle = 'None')
  axline.plot([lineplot[5],lineplot[6]],[lineplot[5],lineplot[6]], color = 'k') #slope of 1
  axline.set_xlim(0,lineplot[5])
  axline.set_ylim(0,lineplot[6])
  axline.set_xticks([lineplot[5],lineplot[6]])
  axline.set_yticks([lineplot[5],lineplot[6]])
  
  #print lineplot

 axScatter.set_xlim( (xlo, xhi) )
 axScatter.set_ylim( (ylo, yhi) )
 yt = axHistx.get_ylim() 
 axHistx.vlines(truth[0],0,yt[1],color='k')
 xt = axHisty.get_xlim()
 axHisty.hlines(truth[1],0,xt[1],color='k')
 
 ntick = len(xtick_in_sub)
 if (ntick > 0):
  axScatter.set_xticks(xtick_in_sub)
  axHistx.set_xticks(xtick_in_sub)
  axHistx.set_xticklabels(['']*ntick)
  axScatter.set_xticklabels(xtick_in_lab)
  axScatter.set_xticks(xtick_in_sub)


  
  axScatter.set_yticks(ytick_in_sub)
  axHisty.set_yticks(ytick_in_sub)
  axHisty.set_yticklabels(['']*ntick) 
  axScatter.set_yticklabels(ytick_in_lab)
 
  #print summary
  xav = np.mean(x)
  yav = np.mean(y)
  
  xsd = np.std(x)
  ysd = np.std(y)
  ysddeg = 1./np.sqrt(1.- yav*yav) * ysd
  
  print 'x info myscatterhist',np.mean(x), np.std(x)
  print 'y info myscatterhist',np.mean(y), np.std(y),180/np.pi*np.mean(y),180/np.pi*ysddeg
  cor_array = np.concatenate((x,y),axis=0)
  print 'nx', nx
  nx = np.shape(x)[0]
  cor_array.resize(2,nx)
  print 'cor-coeff', np.corrcoef(cor_array)
 return axHistx,axScatter








#new function to plt the multiple correlation plots with input from a text file
#if lab style = 'bl', axis tick labels are placed only on xaxis of lower plots and y-axis of left plots

def myscatterhistfile(filename,colhist,nperhist,nvert,nalong,parcols,sublabfile = '',xlim=[], ylim=[], labsin=['',''],labstyle='bl', nsplit = 1, sigconts=[], showconts=[],truth = [666,666], scatterskip=1,nbins=100,spec=[],xtick_in_sub=[],xtick_in_lab=[],ytick_in_sub=[],ytick_in_lab=[],normto1=1,normed=0):
 
 #number of datasets per correlatin plot determined by number of entries in colour list i.e ['r'] just 1 or ['r','b'] two etc 

 
  
 if (nperhist > 1 or nsplit > 1):
  translucency = 0.5
 else:
  translucency = 1.0
 
 with file(filename) as f:
  fname = f.read().splitlines()

 nf  = len(fname)

 taumean_ccf=[]
 taumean_cream=[]
 taumean_real=[]
 tit = []
 #determine if we are making the special ccf plots if ccf and scream data are included
 idx=0
 lineplot_check = 0
 for i in range(nf):
  a = str.split(fname[i], ',')

  fname[i] = a[0]
  
  if (len(a) > 5):
   taumean_ccf.append((a[5], a[6]))
   taumean_cream.append((a[3], a[4]))
   taumean_real.append(a[1])
   
   
  if (len(a) > 1):
   tit.append(a[1])
  else:
   tit.append('')
 
 if (lineplot_check > 0): 
  taumean_ccf   = np.double(np.array(taumean_ccf))
  taumean_cream = np.double(np.array(taumean_cream))
  taumean_real  = np.double(np.array(taumean_real))
 

  taumeanccf_min = np.min(taumean_ccf[:,0] - taumean_ccf[:,1] )
  taumeanccf_max = np.max(taumean_ccf[:,0] + taumean_ccf[:,1] )

  taumeancream_min = np.min(taumean_cream[:,0]- taumean_cream[:,1] )
  taumeancream_max = np.max(taumean_cream[:,0] + taumean_cream[:,1] )
 
  lineplotmin = 0#np.min((taumeancream_min,taumeanccf_min))

  
  lineplotmax = np.max((taumeancream_max,taumeanccf_max, taumean_real[i]))*1.2
  
   
 nsublab = len(sublabfile)
 if (nsublab > 0):
  with file(sublabfile) as f:
   sublab = f.read().splitlines()  
 


 
 dat=[]
 #!load the data
 for i in range(nf):
  print fname[i]
  try:
   a   = np.loadtxt(fname[i])
  except:
   a   = myreadlines.myreadlines(fname[i])
  ncol = a[0,:].shape[0]
  for i2 in range(ncol): #kde code for confidence curves doesn't like nans. Change these to median of data. Dodgy but no one will know!
   cmed = np.median(a[:,i2])
   nancheck = np.isnan(a[:,i2])
   a[nancheck,i2] = cmed
   
  dat.append(a)

 plt.figure(1, figsize=(8,8))
 
 
 labs = ['','']
 for ia in range(nalong):

  if (ia == 0):
   ylab = labsin[1]
  else:
   ylab = ''
  
  for iv in range(nvert):
   if (iv == nvert-1):
    xlab = labsin[0]
   else:
    xlab = ''
   ivert  = iv 
   ialong = ia
   

   for i2 in range(nperhist):
    icol = i2
    ccfcomp = [0,0,0,0,0,0,0]
    if (lineplot_check > 0):
     ccfcomp[0] = taumean_real[idx]
     ccfcomp[1] = taumean_cream[idx][0]
     ccfcomp[2] = taumean_cream[idx][1]
     ccfcomp[3] = taumean_ccf[idx][0]
     ccfcomp[4] = taumean_ccf[idx][1]
     ccfcomp[5] = lineplotmin
     ccfcomp[6] = lineplotmax    


    for isplit in range(nsplit):
     if (nsplit > 1):
      icol = isplit
     else:
      icol = 0
     
     #print idx, len(dat), i2, nperhist
     nd = len(dat[idx])
     npersamp = nd /nsplit
     samp = np.arange(isplit*npersamp,(isplit+1)*npersamp) 
     
     if (showconts[isplit] == 0):
      sigcontsnew = []
     else:
      sigcontsnew = list(sigconts)
     
     print i2,icol, len(colhist), colhist, nperhist, nsplit
     if (colhist[i2][icol] != 'w'):
      if (len(xlim) > 0 ):
       print i2,icol,colhist[i2][icol],nperhist,idx,nf,tit[idx]
       axscat =  myscatterhist_2(dat[idx][samp,parcols[0]],dat[idx][samp,parcols[1]],ialong,ivert,nalong,nvert,xlo = xlim[0], xhi = xlim[1], ylo = ylim[0], yhi = ylim[1],col = colhist[i2][icol], labstyle = labstyle, translucency = translucency, lineplot = ccfcomp,title = tit[idx], special=spec,sigconts=sigcontsnew, truth = truth,xlab=xlab,ylab=ylab,scatterskip=scatterskip,nbins=nbins,xtick_in_sub=xtick_in_sub,ytick_in_sub=ytick_in_sub,xtick_in_lab=xtick_in_lab,ytick_in_lab=ytick_in_lab,normto1=normto1,normed=normed)[1]
      else:
       axscat = myscatterhist_2(dat[idx][samp,parcols[0]],dat[idx][samp,parcols[1]],ialong,ivert,nalong,nvert,col=colhist[i2][icol], labstyle = labstyle, translucency = translucency, lineplot = ccfcomp,title = tit[idx],sigconts=sigcontsnew, truth = truth,xlab=xlab,ylab=ylab,scatterskip=scatterskip,nbins=nbins,special=spec,xtick_in_sub=xtick_in_sub,ytick_in_sub=ytick_in_sub,xtick_in_lab=xtick_in_lab,ytick_in_lab=ytick_in_lab,normto1=normto1,normed=normed)[1]
    #print dat[idx]
    #print dat[idx].min(), dat[idx].max(), np.mean(dat[idx]), idx
    idx = idx + 1
    

 return(axscat)
 



#n = 1000
#nf = 8
#nvert = 4
#nalong =2
#
#colhist=['r']#,'b']
#nperhist = 1
#
#dattest = np.zeros((n,2))
#
#
#fnam = 'histmultidat.dat'
#f = open(fnam,'w')
#for i in range(nf):
# 
# fname = 'test_myscatterhist_'+str(i)+'.dat' +' 1.1 1.0 0.5 1.2 0.5'
# f.write(fname+'\n')
# #ax1 = fig.add_subplot(2,2,i+1)
# dattest[:,0] = np.random.randn(n) + 0.1*i
# dattest[:,1] = np.random.randn(n) + 0.1*i
# np.savetxt(fname,dattest)
#
#f.close()
#
#
#
#ccfc = 'ccfcomp.dat' 
#
#a = myscatterhistfile(fnam,colhist,nvert,nalong,[0,1],xlim = [-5,5], ylim=[-5,5])
#
#
#plt.savefig('figcorplot.png')
#
#


import os
import numpy as np
# this code takes an input light curve
# truncates it into new light curves of length specified by the user.
# resaves the files

# hardwired options
equaldat = 1 #if equaldat = 1, the code will try to pick roughly even chunks but force them all to have the same number of data points

# filter twist th long kepler light curve periodically rotates, if filtertwist = 1, the light curve will be divided into segments which do not have a break in the filter 
filtertwist = 1

dir_default = os.getcwd()
dir = raw_input('Enter directory of data: ')
if not dir:
    dir = dir_default


os.chdir(dir)
fname = raw_input('Enter file name: ')




dat = np.loadtxt(fname)
x   = dat[:,0]
y   = dat[:,1]
sig = dat[:,2]




# manually select the indicies of the x values to chop
idxchophi = [123,1234,2256,34556]
idxchophi = np.array(idxchoplo)
idxchophi  = np.sort(idxchoplo)

nidx = idxchoplo.shape[0]

os.mkdir('mychopandsave_manual')
os.chdir('mychopandsave_manual')

for i in range(nidx):
    idxlo = idxchophi[i]
    idxhi = idxchophi[i+1]
    
    xnew = x[idxlo:idxhi]
    ynew = y[idxlo:idxhi]
    signew = sig[idxlo:idxhi]
    
    nnew = xnew.shape[0]
    
    datnew = np.zeros((nnew,3))
    
    datnew[:,0] = xnew
    datnew[:,1] = ynew
    datnew[:,2] = signew
    
    np.savetxt('chop_'+str(i)+fname, datnew)
    
os.chdir('../')


# function to find the nearest element in an array to some value

import numpy as np

def find_nearest(array,value):
    idx = (np.abs(array-value)).argmin()
    return array[idx]
    
def find_nearest_idx(array,value):
    idx = (np.abs(array-value)).argmin()
    return idx

np.vectorize(find_nearest)    
  
#xin2 and tin2 have fewest points
def find_nearest_fvg(tin1,xin1,tin2,xin2):
    
    xop = find_nearest_vector(xin1,xin2)
    
    # this is the condition for the array having no repeated elements
    if np.array_equal( np.sort(xop), np.unique(xop) ):
        top = tin2
        return(top,xop)
    
    else:
        xdup = xop[ xop[1:] == xop[:-1] ]   #the values of xop which are duplicated
        
        for val in xdup:
            idx = np.where(xop == val)[0]   # identify the indicies of the duplicated values
            idxkeep=[]
            for i in range(t1in):
                idxkeep.append( idx[ np.amin(t1in[i] - t2in[idx]) ] )
                
    # finish the function
    
    return()
                
    
    
    


import kdestats as kde
import numpy as np
import matplotlib.pylab as plt
from scipy import stats
from scipy import interpolate


#test script how to identify confidence regions from a kde useful for outlier code!

covmat = [[1., 11.5], [1.5, 4.]]

#fop_pca = np.random.multivariate_normal([-3, 5], covmat, 10000)

a = np.random.multivariate_normal([-3, 5], covmat, 5000)
b = np.random.multivariate_normal([0, 0], covmat, 5000)
fop_pca = np.vstack((a,b))



xmin,xmax = np.min(fop_pca[:,0]),np.max(fop_pca[:,0])
ymin,ymax = np.min(fop_pca[:,1]),np.max(fop_pca[:,1])
X, Y = np.mgrid[xmin:xmax:100j, ymin:ymax:100j]
positions = np.vstack([X.ravel(), Y.ravel()])
values = fop_pca.T
kernel = stats.gaussian_kde(values)
Z = np.reshape(kernel(positions).T, X.shape)



#kdehist = kde.kdehist2(xy[:,0], xy[:,1], [30, 30])
clevels = kde.confmap(Z.T, [.9973,.9545,.6827])



fig = plt.figure()
ax1 = fig.add_subplot(111)

xmod = X[:,0]
ymod = Y[0,:]
nxmod = np.shape(xmod)[0]
#ax1.contour(xmod,ymod,Z,clevels)
ax1.scatter(fop_pca[:,0],fop_pca[:,1])

#f = interpolate.interp2d(xmod, ymod, Z, kind='cubic')
#zdat = f(values[0,:],values[1,:])
#find Z values of actual scatter points
idx_x = np.array(np.interp(values[0,:],xmod,np.arange(nxmod)),dtype='int')
idx_y = np.array(np.interp(values[1,:],ymod,np.arange(nxmod)),dtype='int')
zdat = Z[idx_x,idx_y]


for i in range(len(clevels)):
 idlike = np.where(zdat > clevels[i])
 ax1.scatter(fop_pca[idlike,0],fop_pca[idlike,1])
plt.show()






import numpy as np



def tlc(embh,rld):
 rm = 2.58e13*rld
 rsc = 3000*embh
 c   = 2.9979e8
 tday = rm/c

 return(tday)




def tdyn(embh,rld):
 gnewt = 6.673e-11
 rm    = rld*2.58e13
 rm3 = rm**3
 
 emkg = 2.e30*embh
 return(np.sqrt(rm3/(gnewt*emkg)))

def tth(embh,rld,alpha=0.03):
 tdy = tdyn(embh,rld)
 return(tdy/alpha)
 

def tvisc(embh,rld,alpha=0.03):
 
 rm = 2.58e13*rld
 rcm = rm*100
 r18 = rcm/1.e18
 secyr = 24*3600*365.25
 return(secyr * 10**10 * (alpha/0.03) * r18**1.5)
 
 
 
 
 
 
 
 

embh = 1.e8
rld = 1
secday = 86400

print 'tlc, tdyn, tth, tvisc'

tl = tlc(embh,rld)/secday
tt = tth(embh,rld)/secday
tv = tvisc(embh,rld)/secday
td  = tdyn(embh,rld)/secday

print tl,td,tt,tv






import vortexa_makefake as vmf
import numpy as np

#dat ntimes x 2 matrix first column is the driver second is the response
def rli(dat):

 n = len(dat[:,0])
 npsi = n
 dmk = np.mean(dat[:,0])
 drive = dat[:,0]
 #smoothing matrix (see thesis page 123)
 s = np.zeros((n,npsi))
 for ik in range(n):
  if (ik == 0):
   s[ik,0] = 2
  elif (ik == 1):
   s[ik,1] = 2
  elif (ik == n-1):
   s[ik,-1] = 2
  elif (ik == n-2):
   s[ik,-2] = 2
  else:
   s[ik,ik-2:ik+3] = np.array([0.5,-2,3,-2,0.5])


 #driving matrix of look back times
 dmat = np.zeros((n,npsi))
 for ik in range(n):
  ik2 = np.arange(npsi)
  ilb = np.arange(ik,ik-npsi,-1)
  ilb[ilb < 0] = 0
  dmat[ik,ik2]=drive[ilb]
  #for ik2 in range(npsi):
  # idx = max(ik - ik2,0)
  # dmat[ik,ik2] = drive[idx]



#now construct hessian matrix from dmat and sigma 2 see also page 123 of thesis

 return()


#for ik in range(npsi):
# for ij in range(1,npsi):
#
##sum = 0.0
#  for it3 in range(1,nf):
#   sum = sum + (xitp(it3,ij) - dmj)*(xitp(it3,ik) - dmk)/sig2(it3)
#!write(*,*) it3,xitp(it3,ij),xitp(it3,ik),sig2(it3)
#enddo
#hesmat(ij,ik) = sum
#
#
#sum = 0.d0
#!now add constraint from smoothness function
#if ((ik .gt. 2) .and. (ik .lt. npsi-1)) then
# if ((ij .eq. ik - 2) .or. (ij .eq. ik + 2)) then
# sum = 0.25   !0.5
# else if ((ij .eq. ik -1) .or. (ij .eq. ik + 1)) then
# sum = -1.        !*2
# else if (ij .eq. ik) then
# sum = 1.5    !3
# endif
# 
# 
# !if (ij .eq. ik - 1) then 
# !sum = sum - alpha*0.5
# !else if (ij .eq. ik) then 
# !sum = sum + alpha*1.
# !else if (ij .eq. ik + 1) then
# !sum = sum - alpha*0.5
# !else
# !sum = sum - alpha*0.
# !endif
#
#else if ((ik .le. 2) .or. (ik .ge. npsi - 1)) then
# if (ij .eq. ik) then
# sum = 1. !+ alpha*1.!2
# else
# sum = 0.
# endif
#endif
#
#!if (ij .eq. 1) then  !! edge effects. This minimises psi^2 for the edges to pull tf down to 0 at ends
#!sum = sum - 0.0!1.e6*alpha*1.
#!else
#!sum = sum + alpha*0.
#!endif
#!else if (ik .eq. npsi) then
#!if (ij .eq. npsi) then
#!sum = sum - 0.0!1.e6*alpha*1.
#!else
#!sum = sum + alpha*0.
#!endif
#
#!endif
#!else if (ik .eq. 1) then !what to do about edge effects (ignore for now)
#!if (ij .eq. 1) then
#!sum = sum + alpha*1.
#!else if (ij .eq. 2) then
#!sum = sum + alpha*
#!endif
#
#smoothmat(ij,ik) = sum
#
#
#
#!write(*,*) ij,ik,sum
#!read(*,*)
#enddo














tlo = 0.0
thi = 100.0
dt = 0.01

a = vmf.makefake(tlo,thi,dt,shift = [2,3],noise=0.1)


din = np.array([a[0].values[:,1],a[1].values[:,1]]).T
rli(din)






import numpy as np
import scipy
import scipy.stats


#randomly sample nsamp from a distribution in bins of width binwid
#if binwidin -ve the absolute value specifies number of bins rather than the width
#if specbw 1, then alter bin widths to ensure equal samples in each bin
def myransamp(dist,nsamp,binwidin,dloin='',dhiin='',specbw = 1):
 
 if (specbw == 1):
  print 'myransamp.py adaptive bin size ON'
 
 idxout = []
 binout = []
 if (dloin == ''):
  dlo = np.min(dist)
 else:
  dlo = dloin
  
 if (dhiin == ''):
  dhi = np.max(dist)
 else:
  dhi = dhiin

 blosave = []
 bwsave = []
#one option specifies binwidth the other specifies bin number with option to equalize number of objects in each bin 
 if (binwidin < 0):
  nbin = np.abs(binwidin)
  for i in range(nbin):
   if (specbw == 1):
    dsort = np.sort(dist)
    ndist = np.shape(dist)[0]
    nb = int(np.floor(ndist/nbin)) 
    blonow = dsort[i*nb]
    blosave.append(blonow)
    if (i < nbin - 1):
     blonext = dsort[(i+1)*nb]
     bwsave.append(blonext - blonow)
    else:
     bwsave.append(dhi - blonow)   
   else:
    binwid = (dhi - dlo)/(nbin - 1)
    blosave.append(dlo + i*binwid)   
    bwsave.append(binwid)
    
 else:
  binwid = binwidin
  nbin = np.floor(dhi-dlo)/binwid + 1
  for i in range(nbin):
   blosave.append(dlo + binwid*i)
   bwsave.append(binwid)
    
  
 nperbin = nsamp /nbin 
 
 for i in range(nbin):
  blonow = blosave[i]#dlo + i*binwidth
  bhinow = blonow + bwsave[i]#blonow + binwidth
  idxsamp = np.where((dist > blonow) & (dist < bhinow))[0]
  
  nss = np.shape(idxsamp)[0]
  nnow = min(nss,nperbin)
  if (nnow < nperbin):
   print 'bins too fine for number of requested samples per bin, picking smaller sample size',nnow,nperbin
  idbin = np.random.choice(idxsamp,size=nnow,replace=False)
  idxout.append(idbin)
  binout.append(blonow)
  
 return(blosave,bwsave,idxout)
  
  
  
  
  
  
  
  
  
#this code will bin the data into a 2D histogram of approximately equal 
#bin sizes, then randomly sample nsamp  
#minbin roughly how many samples you want per bin
#datout should be a list of approximately uniformly sampled dat

#if nbinin > 0 then this specifies the number of linearly spaced bins in each dimension
#from the lowest to highest data point

#dlimin = [[dlo,dhi],[dlo,dhi].....] for each dimension limits for each bin
#... else if -1 use minimum and maximum point 
#...else use dlminin*nsd about mean either side

def myransampnd(dat,nsamp,minbin=10,nbinin = -1,dlminin=-1):
 ndim = np.shape(dat[0,:])[0]
 ndat = np.shape(dat[:,0])[0]
 datout = []
 if (isinstance(dlminin,list)):
  dlo = [dlminin[i][0] for i in range(ndim)]
  dhi = [dlminin[i][1] for i in range(ndim)]
 elif (dlminin == -1):
  dlo = [np.min(dat[:,i]) for i in range(ndim)] 
  dhi = [np.max(dat[:,i]) for i in range(ndim)] 
 elif (dlminin > 0):
  dlo = [np.mean(dat[:,i]) - dlminin*np.std(dat[:,i]) for i in range(ndim)] 
  dhi = [np.mean(dat[:,i]) + dlminin*np.std(dat[:,i]) for i in range(ndim)] 

 #need fewer bins than samples I think this should give about minbin samples per bin
 if (nbinin == -1):
  nbin = nsamp**(1./ndim)/minbin
 else:
  nbin = nbinin
  
 
 #decide bin sizes
 bins = []
 idxselect = []
 for i in range(ndim):
  if (nbin == -1):
   ddim = dat[:,i]
   dsort = np.sort(ddim)
   dbin = dsort[::minbin]
  else:
   dbin = np.linspace(dlo[i],dhi[i],nbin)
   
  bins.append(dbin) 
  #idxselect = 
  
 bins = np.array(bins)
 print np.shape(bins)
 print np.shape(dat)
 dhist = scipy.stats.binned_statistic_dd(dat,np.ones(ndat),bins=bins)
 idbin = dhist[-1]
 idbinu = np.unique(idbin)
 
 
 print np.shape(idbinu)
 print np.shape(idbin)
 for idnow in idbinu:
  try:
   idxu = np.where(idbin == idnow)[0]
   nnow = 1.0#min(np.shape(idxu)[0],minbin)
   idpick = np.random.choice(idxu,size=nnow,replace=False)
   datout.append(dat[idpick,:])
  except:
   continue
 
 return(datout) 
  
 #np.histogramdd(dat,bins=bins)



#myransamp(dist,nsamp,binwidin,dloin='',dhiin='',specbw = 1):



import numpy as np
import os



dir = '/Users/ds207/Documents/standrews/sta/fort/fortcode/mcmcmulti3/aas_rebinned/'
file_list = 'mcmcmultinames.dat'
tlo = 6690.0
thi = 6810.0



pwd = os.getcwd()
os.chdir(dir)
f = open(file_list)
files = f.read().splitlines()
f.close()
nf = len(files)


for fnow in files:
 print 'reading ',fnow
 dat = np.loadtxt(fnow)
 idxtrue = ((dat[:,0] >= tlo) & (dat[:,0] <= thi))
 datnew = dat[idxtrue,:]
 
 np.savetxt('extracted_'+fnow,datnew)
 

os.chdir(pwd) 
 


import numpy as np
from mylcgen import *
from myrandom import *
import matplotlib.pylab as plt



#function to fit the sine or cosine transformastion to an input light curve
#parameters: 
#time,y,sig the data arrays
# freq if 1 then use flo and fhi to obtain a suitable array of frequencies to calc the transform
# flo,fhi if -ve (and freq == 1) then go to abs(fhi) x the nyquist frequency for high frequency and abs(flo) times the repeat period for low frequency, df = low frequency
# dtres if -ve then go to abs(dtres) x the average time resolution
# costrans if 1 calcuate the cosine transform else (if 0) do the sine transformation

def sct(time,y,sig,freq=np.zeros(0),flo=-0.5,fhi=-2,dtres=-10,costrans=1,ploton = 1):
 
 #preamble !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 twopi = 2*np.pi
 tlen  = time[-1] - time[0]
 dt    = np.median(time[1:] - time[:-1])
 ndat  = np.shape(y)[0]
 tlo   = np.min(time)
 thi   = np.max(time)
 
 if (dtres < 0):
  dtresplot = dt/np.abs(dtres)
 else: 
  dtresplot = dtres
  
 #generate angular frequencies
 if (np.shape(freq)[0] == 0):
  wlo = twopi*flo
  whi = twopi*fhi
  if (flo <0):
   wlo = twopi/tlen*np.abs(flo)
  if (fhi < 0):
   whi = twopi/dt * np.abs(fhi)
  w = np.arange(wlo,whi+wlo,wlo)
 else:
  w = twopi*freq
 
 #end of preamble!!!!!!!!!!!!!!!!!!!!!!!!!!!
 if (costrans == 1):
  phase = 0
 elif (costrans == 0):
  phase = -np.pi/2
 
 
 #now compute the hessian matrix
 nw = np.shape(w)[0]
 np2 = 2*nw
 
 cvec = np.ones(nw)
 hes = np.ones((nw,nw))
 sig2 = sig*sig
 #save the sine and cosine value in  2d array
 cwt = np.ones((ndat,nw))
 swt = np.ones((ndat,nw))
 for iw in range(nw):
  cwt[:,iw] = np.cos(w[iw]*time +phase)
 
 for ikp in range(nw):
  wp = w[ikp]
  
  wp2 = wp*wp
  cvec[ikp]   = np.sum(y*cwt[:,ikp]/sig2)
  #print ikp,nw
  for ik in range(nw):
   #cosine column
   hnow_c = np.sum(cwt[:,ik]*cwt[:,ikp]/sig2)
   if (ik == ikp):
    hnow_c = hnow_c + wp2
   hes[ikp,ik]   = hnow_c
 
    
 
       
 #invert the matrix to get the covariance matrix
 cov = np.linalg.inv(hes)
 
 
 #find the parameters
 parm  = cov.dot(cvec)
 ckout = parm
 
 
 
 
 
 
 
 
 
 #plot the input and output light curves to see if they match
 fout = w/twopi
 
 tplot = np.arange(tlo,thi+dtresplot,dtresplot)
 xplot = []
 for tnow in tplot:
  xplot.append( np.sum(ckout*np.cos(w*tnow + phase) ) )
 
 if (ploton == 1):
  fig = plt.figure()
  ax1 = fig.add_subplot(211) 
  ax1.errorbar(time,y,sig,ls='',color='k')
  if (costrans==1):
   ax1.plot(tplot,xplot,color='b',label='cosine fit')
  else:
   ax1.plot(tplot,xplot,color='b',label='sine fit')
  ax1.set_xlabel('Time (days)')
  ax1.set_ylabel('flux')
  plt.legend()
  
  ax2 = fig.add_subplot(212) 
  ax2.plot(fout,ckout**2,ls='',marker='o')
  ax2.set_xscale('log')
  ax2.set_yscale('log')
  ax2.set_xlabel('Frequency (cyc/day)')
  ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
  plt.savefig('test_r_hesfit_sine_cosine.pdf')
 
 
 
 return(fout, parm, tplot, xplot)
 


import numpy as np
#convolve xd by tf 1d


def myconvolve(xd,tf):

 nmodtf = np.shape(tf)[0]
 nd     = np.shape(xd)[0]
 #manual convolution (i dont know how to use the python one!)
 modf= []
 for i in range(nd):
  xnow = xd[i]
  idxlo = max(0,i-nmodtf)
  idxhi = i
  idxd = np.arange(idxlo,idxhi,1)[::-1]
  idxtf = np.arange(idxhi  - idxlo)
  try:
   modf.append( np.sum(xd[idxd]*tf[idxtf]) )
  except:
   modf.append(0)
 modf = np.array(modf)
 return(modf)
 
 

def mc3(td,xd,tau,tf):



 nmodtf = np.shape(tf)[0]
 nd     = np.shape(xd)[0]
 #manual convolution (i dont know how to use the python one!)
 modf= []
 
 taulo = np.min(tau)
 tauhi = np.max(tau)
 dtau  = np.mean(tau[1:]-tau[:-1])
 ntau  = np.shape(tau)[0]

 dtgrid = np.mean(td[1:]-td[:-1])
 #check if driver and tau grid are on same scale, interpolate tf if not
 
 if (dtau != dtgrid):
  #print('myconvolve response and driver not on same time grid, interpolating response function.',dtgrid,dtau)
  tauint = np.arange(taulo,tauhi+dtgrid,dtgrid)
  tfint  = np.interp(tauint,tau,tf) 
 else:
  tauint = tau
  tfint = tf
 
 itflo = int(np.floor(taulo/dtgrid))
 itfhi = int(np.floor(tauhi/dtgrid))
 
 
 for i in range(nd):
  xnow = xd[i]
  idxlo = max(0,i-itfhi)
  idxhi = min(nd-1,i-itflo)
  idxd = np.arange(idxlo,idxhi,1)[::-1]
  idxtf = np.arange(idxhi  - idxlo)
  #print np.shape(idxtf),np.shape(idxd),np.sum(xd[idxd]*tf[idxtf])
  try:
   modf.append( np.sum(xd[idxd]*tfint[idxtf]) )
  except:
   modf.append(0)
 modf = np.array(modf)
 #print modf
 #raw_input()
 return(modf)